@@ -2,7 +2,7 @@
 
 #export DYLD_LIBRARY_PATH=$HOME/ffmpeg-2.2.3/lib:$DYLD_LIBRARY_PATH
 #export DYLD_LIBRARY_PATH=$HOME/ffmpeg-2.8/lib:$DYLD_LIBRARY_PATH
-#export DYLD_LIBRARY_PATH=/usr/local/Cellar/ffmpeg/4.1.2/lib:$DYLD_LIBRARY_PATH
+export DYLD_LIBRARY_PATH=/usr/local/Cellar/ffmpeg/6.0/lib:$DYLD_LIBRARY_PATH
 
 #JAVA_HOME=`/usr/libexec/java_home`
 #JAVA_HOME=`/usr/libexec/java_home -version 1.8`
