@@ -1,5 +1,5 @@
 /**
- * Copyright 2011 JogAmp Community. All rights reserved.
+ * Copyright 2011-2023 JogAmp Community. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are
  * permitted provided that the following conditions are met:
@@ -33,17 +33,22 @@ import com.jogamp.opengl.GL;
 import com.jogamp.opengl.GL2ES2;
 import com.jogamp.opengl.GLException;
 import com.jogamp.opengl.GLUniformData;
+import com.jogamp.opengl.math.Vec4f;
 
 import jogamp.common.os.PlatformPropsImpl;
 import jogamp.graph.curve.opengl.shader.UniformNames;
 
-import com.jogamp.common.os.Platform;
 import com.jogamp.graph.curve.Region;
-import com.jogamp.graph.geom.Vertex;
-import com.jogamp.opengl.util.GLArrayDataServer;
+import com.jogamp.opengl.util.GLArrayDataWrapper;
 import com.jogamp.opengl.util.PMVMatrix;
 import com.jogamp.opengl.util.glsl.ShaderProgram;
 
+/**
+ * The RenderState is owned by {@link RegionRenderer}.
+ *
+ * It holds rendering state data like {@link PMVMatrix}, viewport,
+ * but also the current {@link #getColorStatic(float[]) static color}.
+ */
 public class RenderState {
     private static final String thisKey = "jogamp.graph.curve.RenderState" ;
 
@@ -82,19 +87,10 @@ public class RenderState {
      */
     public static final int BITHINT_GLOBAL_DEPTH_TEST_ENABLED = 1 << 1 ;
 
-    public static RenderState createRenderState(final Vertex.Factory<? extends Vertex> pointFactory) {
-        return new RenderState(pointFactory, null);
-    }
-
-    public static RenderState createRenderState(final Vertex.Factory<? extends Vertex> pointFactory, final PMVMatrix pmvMatrix) {
-        return new RenderState(pointFactory, pmvMatrix);
-    }
-
     public static final RenderState getRenderState(final GL2ES2 gl) {
         return (RenderState) gl.getContext().getAttachedObject(thisKey);
     }
 
-    private final Vertex.Factory<? extends Vertex> vertexFactory;
     private final PMVMatrix pmvMatrix;
     private final float[] weight;
     private final FloatBuffer weightBuffer;
@@ -110,7 +106,7 @@ public class RenderState {
     private static int nextID = 1;
 
     /**
-     * Representation of {@link RenderState} data for one {@link ShaderProgram}
+     * Representation of {@link RenderState} data per {@link ShaderProgram}
      * as {@link GLUniformData}.
      * <p>
      * FIXME: Utilize 'ARB_Uniform_Buffer_Object' where available!
@@ -143,7 +139,8 @@ public class RenderState {
          */
         public final boolean update(final GL2ES2 gl, final RenderState rs, final boolean updateLocation, final int renderModes, final boolean pass1, final boolean throwOnError) {
             if( rs.id() != rsId ) {
-                gcu_PMVMatrix01.setData(rs.pmvMatrix.glGetPMvMatrixf());
+                // Assignment of Renderstate buffers to uniforms (no copy, direct reference)
+                gcu_PMVMatrix01.setData(rs.pmvMatrix.getSyncPMvMat());
                 gcu_Weight.setData(rs.weightBuffer);
                 gcu_ColorStatic.setData(rs.colorStaticBuffer);
                 rsId = rs.id();
@@ -186,27 +183,36 @@ public class RenderState {
         }
     }
 
-    protected RenderState(final Vertex.Factory<? extends Vertex> vertexFactory, final PMVMatrix pmvMatrix) {
+    /**
+     * Create a RenderState, a composition of RegionRenderer
+     * @param sharedPMVMatrix optional shared PMVMatrix, if null using a local instance
+     */
+    /* pp */ RenderState(final PMVMatrix sharedPMVMatrix) {
         this.id = getNextID();
         this.sp = null;
-        this.vertexFactory = vertexFactory;
-        this.pmvMatrix = null != pmvMatrix ? pmvMatrix : new PMVMatrix();
+        this.pmvMatrix = null != sharedPMVMatrix ? sharedPMVMatrix : new PMVMatrix();
         this.weight = new float[1];
         this.weightBuffer = FloatBuffer.wrap(weight);
-        this.colorStatic = new float[4];
+        this.colorStatic = new float[] { 1, 1, 1, 1 };
         this.colorStaticBuffer = FloatBuffer.wrap(colorStatic);
         this.hintBitfield = 0;
     }
 
     public final int id() { return id; }
+
+    /** Return the current {@link ShaderProgram} */
     public final ShaderProgram getShaderProgram() { return sp; }
+
+    /** Return whether the current {@link ShaderProgram} is {@link ShaderProgram#inUse() in use}. */
     public final boolean isShaderProgramInUse() { return null != sp ? sp.inUse() : false; }
 
     /**
-     * Set a {@link ShaderProgram} and enable it. If the given {@link ShaderProgram} is new,
-     * method returns true, otherwise false.
+     * Sets the current {@link ShaderProgram} and enables it.
+     *
+     * If the given {@link ShaderProgram} is not {@link #getShaderProgram() the current}, method returns true, otherwise false.
+     *
      * @param gl
-     * @param spNext
+     * @param spNext the next current {@link ShaderProgram} to be set and enabled
      * @return true if a new shader program is being used and hence external uniform-data and -location,
      *         as well as the attribute-location must be updated, otherwise false.
      */
@@ -223,8 +229,6 @@ public class RenderState {
         return true;
     }
 
-    public final Vertex.Factory<? extends Vertex> getVertexFactory() { return vertexFactory; }
-
     public final PMVMatrix getMatrix() { return pmvMatrix; }
 
     public static boolean isWeightValid(final float v) {
@@ -238,10 +242,14 @@ public class RenderState {
         weight[0] = v;
     }
 
-
-    public final float[] getColorStatic(final float[] rgbaColor) {
-        System.arraycopy(colorStatic, 0, rgbaColor, 0, 4);
-        return rgbaColor;
+    public final Vec4f getColorStatic(final Vec4f rgbaColor) {
+        return rgbaColor.set(colorStatic);
+    }
+    public final void setColorStatic(final Vec4f rgbaColor){
+        colorStatic[0] = rgbaColor.x();
+        colorStatic[1] = rgbaColor.y();
+        colorStatic[2] = rgbaColor.z();
+        colorStatic[3] = rgbaColor.w();
     }
     public final void setColorStatic(final float r, final float g, final float b, final float a){
         colorStatic[0] = r;
@@ -263,7 +271,8 @@ public class RenderState {
         if( updateLocation || 0 > data.getLocation() ) {
             final boolean ok = 0 <= data.setLocation(gl, sp.program());
             if( throwOnError && !ok ) {
-                throw new GLException("Could not locate "+data);
+                sp.dumpSource(System.err);
+                throw new GLException("Could not locate "+data.getName()+" in "+sp+", "+data);
             }
             return ok;
         } else {
@@ -285,7 +294,8 @@ public class RenderState {
         if( updateLocation ) {
             updateData = 0 <= data.setLocation(gl, sp.program());
             if( throwOnError && !updateData ) {
-                throw new GLException("Could not locate "+data);
+                sp.dumpSource(System.err);
+                throw new GLException("Could not locate "+data.getName()+" in "+sp+", "+data);
             }
         }
         if( updateData ){
@@ -302,11 +312,12 @@ public class RenderState {
      * @param throwOnError TODO
      * @return true if no error occured, i.e. all locations found, otherwise false.
      */
-    public final boolean updateAttributeLoc(final GL2ES2 gl, final boolean updateLocation, final GLArrayDataServer data, final boolean throwOnError) {
+    public final boolean updateAttributeLoc(final GL2ES2 gl, final boolean updateLocation, final GLArrayDataWrapper data, final boolean throwOnError) {
         if( updateLocation || 0 > data.getLocation() ) {
             final boolean ok = 0 <= data.setLocation(gl, sp.program());
             if( throwOnError && !ok ) {
-                throw new GLException("Could not locate "+data);
+                sp.dumpSource(System.err);
+                throw new GLException("Could not locate "+data.getName()+" in "+sp+", "+data);
             }
             return ok;
         } else {
@@ -325,11 +336,11 @@ public class RenderState {
         hintBitfield &= ~mask;
     }
 
-    public void destroy(final GL2ES2 gl) {
-        if( null != sp ) {
-            sp.destroy(gl);
-            sp = null;
-        }
+    /**
+     * Only nullifies {@link ShaderProgram} reference owned by {@link RegionRenderer}.
+     */
+    /* pp */ void destroy() {
+        sp = null; // owned by RegionRenderer
     }
 
     public final RenderState attachTo(final GL2ES2 gl) {
