@@ -1,5 +1,5 @@
 /**
- * Copyright 2012 JogAmp Community. All rights reserved.
+ * Copyright 2012 - 2019 JogAmp Community. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are
  * permitted provided that the following conditions are met:
@@ -27,12 +27,14 @@
  */
 package com.jogamp.opengl;
 
+import java.lang.reflect.Field;
 import java.util.IdentityHashMap;
 
 import com.jogamp.nativewindow.AbstractGraphicsDevice;
 import com.jogamp.opengl.GLCapabilitiesImmutable;
 
 import com.jogamp.common.os.Platform;
+import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.opengl.egl.EGL;
 import com.jogamp.opengl.egl.EGLExt;
 
@@ -47,10 +49,46 @@ import com.jogamp.opengl.egl.EGLExt;
  * <i>Some</i> <code>GL_VENDOR</code> and <code>GL_RENDERER</code> strings are
  * listed here <http://feedback.wildfiregames.com/report/opengl/feature/GL_VENDOR>.
  * </p>
+ * <p>
+ * For testing purpose or otherwise, you may override the implemented
+ * quirk bit setting behavior using {@link GLRendererQuirks.Override}.
+ * </p>
  */
 public class GLRendererQuirks {
     /**
-     * Crashes XServer when using double buffered PBuffer with GL_RENDERER:
+     * Allow overriding any quirk settings
+     * via the two properties:
+     * <ul>
+     *   <li>jogl.quirks.force</li>
+     *   <li>jogl.quirks.ignore</li>
+     * </ul>
+     * Both contain a list of capital sensitive quirk names separated by comma.
+     * Example:
+     * <pre>
+     * java -Djogl.quirks.force=GL3CompatNonCompliant,NoFullFBOSupport -cp my_classpath some.main.Class
+     * </pre>
+     * <p>
+     * Naturally, one quirk can only be listed in one override list.
+     * Hence the two override sets force and ignore are unique.
+     * </p>
+     */
+    public static enum Override {
+        /**
+         * No override.
+         */
+        NONE,
+        /**
+         * Enforce the quirk, i.e. allowing the code path to be injected w/o actual cause.
+         */
+        FORCE,
+        /**
+         * Ignore the quirk, i.e. don't set the quirk if otherwise caused.
+         */
+        IGNORE
+    }
+
+    /**
+     * Crashes XServer when using double buffered PBuffer with hardware GL_RENDERER on Mesa < 18.2.2:
      * <ul>
      *  <li>Mesa DRI Intel(R) Sandybridge Desktop</li>
      *  <li>Mesa DRI Intel(R) Ivybridge Mobile - 3.0 Mesa 8.0.4</li>
@@ -69,7 +107,7 @@ public class GLRendererQuirks {
     /** No offscreen bitmap available, currently true for JOGL's OSX implementation. */
     public static final int NoOffscreenBitmap       = 3;
 
-    /** SIGSEGV on setSwapInterval() after changing the context's drawable w/ 'Mesa 8.0.4' dri2SetSwapInterval/DRI2 (soft & intel) */
+    /** SIGSEGV on setSwapInterval() after changing the context's drawable w/ Mesa >= 8.0.4 until Mesa < 18.2.2: dri2SetSwapInterval/DRI2 (soft & intel) */
     public static final int NoSetSwapIntervalPostRetarget = 4;
 
     /**
@@ -82,12 +120,14 @@ public class GLRendererQuirks {
     public static final int GLSLBuggyDiscard = 5;
 
     /**
-     * Non compliant GL context due to a buggy implementation not suitable for use.
+     * Non compliant OpenGL 3.1+ compatibility profile due to a buggy implementation not suitable for use.
      * <p>
-     * Currently, Mesa >= 9.1.3 (may extend back as far as 9.0) OpenGL 3.1 compatibility
-     * context is not compliant. Most programs will give completely broken output (or no
-     * output at all. For now, this context is not trusted.
+     * Mesa versions in the range [9.1.3 .. 18.2.0[ are not fully compliant with the
+     * OpenGL 3.1 compatibility profile.
+     * Most programs will give completely broken output (or no
+     * output at all.
      * </p>
+     * <p>
      * The above has been confirmed for the following Mesa 9.* GL_RENDERER strings:
      * <ul>
      *   <li>Mesa .* Intel(R) Sandybridge Desktop</li>
@@ -95,9 +135,10 @@ public class GLRendererQuirks {
      * </ul>
      * </p>
      * <p>
-     * It still has to be verified whether the AMD OpenGL 3.1 core driver is compliant enought.
+     * Default implementation sets this quirk on all Mesa < 18.2.0 drivers.
+     * </p>
      */
-    public static final int GLNonCompliant = 6;
+    public static final int GL3CompatNonCompliant = 6;
 
     /**
      * The OpenGL context needs a <code>glFlush()</code> before releasing it, otherwise driver may freeze:
@@ -191,10 +232,7 @@ public class GLRendererQuirks {
      *     </ul></li>
      * </ul>
      * <p>
-     * Also enabled via {@link #BuggyColorRenderbuffer}.
-     * </p>
-     * <p>
-     * Quirk can also be enabled via property: <code>jogl.fbo.force.min</code>.
+     * Note: Also enabled via {@link #BuggyColorRenderbuffer}.
      * </p>
      */
     public static final int NoFullFBOSupport = 11;
@@ -335,9 +373,6 @@ public class GLRendererQuirks {
      * <p>
      * Note: GLFBODrawable always uses texture attachments if set.
      * </p>
-     * <p>
-     * Quirk can also be enabled via property: <code>jogl.fbo.force.nocolorrenderbuffer</code>.
-     * </p>
      */
     public static final int BuggyColorRenderbuffer  = 18;
 
@@ -347,7 +382,7 @@ public class GLRendererQuirks {
      * <p>
      * Some drivers wrongly claim to support pbuffers
      * with accumulation buffers. However, the creation of such pbuffer fails:
-     * <pre>
+      * <pre>
      *   com.jogamp.opengl.GLException: pbuffer creation error: Couldn't find a suitable pixel format
      * </pre>
      * </p>
@@ -440,6 +475,7 @@ public class GLRendererQuirks {
      *   <li>GNU/Linux X11 Nvidia proprietary driver
      *   <ul>
      *     <li>GL_VERSION      4.4.0 NVIDIA 340.24</li>
+     *     <li>GL_VERSION      4.6.0 NVIDIA 440.36</li>
      *     <li>Platform        GNU/Linux X11</li>
      *   </ul></li>
      * </ul>
@@ -447,17 +483,71 @@ public class GLRendererQuirks {
      */
     public static final int NoSurfacelessCtx = 22;
 
-    /** Return the number of known quirks. */
-    public static final int getCount() { return 23; }
+    /**
+     * No FBO support at all.
+     * <p>
+     * This quirk currently exist to be injected by the user via the properties only,
+     * see {@link GLRendererQuirks.Override}.
+     * </p>
+     */
+    public static final int NoFBOSupport = 23;
+
+    /**
+     * Don't use the ChooseFBConfig's best match,
+     * instead utilize the given {@link GLCapabilitiesChooser} or {@link DefaultGLCapabilitiesChooser}
+     * without any recommendation.
+     * <p>
+     * The default behavior without this quirk is using a given {@link GLCapabilitiesChooser}
+     * and pass the ChooseFBConfig's best match as a recommendation.
+     * </p>
+     * <p>
+     * This quirk currently exist to be injected by the user via the properties,
+     * see {@link GLRendererQuirks.Override}.
+     * </p>
+     */
+    public static final int DontChooseFBConfigBestMatch = 24;
+
+    /**
+     * On Mesa >= 18.0.0, {@code glXChooseFBConfig} selects <i>better</i>
+     * {@link GLCapabilities} FBConfig than actually supported by
+     * {@link glXCreatePbuffer} and {@code glXCreateGLXPixmap}.
+     * <p>
+     * As tested on Mesa 18.3.6, requesting an RGB 8bit color component
+     * FBConfig for {@code GLX_PBUFFER_BIT} and {@code GLX_PIXMAP_BIT} {@code GLX_DRAWABLE_TYPE}s
+     * via {@code glXChooseFBConfig} returns an RGB 10bit color component
+     * FBConfig as its best match.
+     * Subsequent {@code glXCreatePbuffer} and {@code glXCreateGLXPixmap} calls fail.
+     * </p>
+     * <p>
+     * This bugs seems to occur in Mesa >= 18.0.0 using <i>allow_rgb10_configs</i>, which is the default now.
+     * While the 10 bit color components are not listed for
+     * on-screen {@code GLX.GLX_WINDOW_BIT} {@code GLX_DRAWABLE_TYPE}s,
+     * they are listed for above mentioned off-screen types without {@code XVisualInfo} reference.
+     * </p>
+     * <p>
+     * This quirk disables using any color component > 8 bit for
+     * {@code GLX_PBUFFER_BIT} and {@code GLX_PIXMAP_BIT} types
+     * and forces using an optional given {@link GLCapabilitiesChooser}
+     * or the {@link DefaultGLCapabilitiesChooser}.
+     * </p>
+     * <p>
+     * Note: Also implies {@link #DontChooseFBConfigBestMatch} for {@code GLX_PBUFFER_BIT} and {@code GLX_PIXMAP_BIT} types.
+     * </p>
+     */
+    public static final int No10BitColorCompOffscreen = 25;
+
+    /** Return the number of known quirks, aka quirk bit count. */
+    public static final int getCount() { return 26; }
 
     private static final String[] _names = new String[] { "NoDoubleBufferedPBuffer", "NoDoubleBufferedBitmap", "NoSetSwapInterval",
                                                           "NoOffscreenBitmap", "NoSetSwapIntervalPostRetarget", "GLSLBuggyDiscard",
-                                                          "GLNonCompliant", "GLFlushBeforeRelease", "DontCloseX11Display",
+                                                          "GL3CompatNonCompliant", "GLFlushBeforeRelease", "DontCloseX11Display",
                                                           "NeedCurrCtx4ARBPixFmtQueries", "NeedCurrCtx4ARBCreateContext",
                                                           "NoFullFBOSupport", "GLSLNonCompliant", "GL4NeedsGL3Request",
                                                           "GLSharedContextBuggy", "GLES3ViaEGLES2Config", "SingletonEGLDisplayOnly",
                                                           "NoMultiSamplingBuffers", "BuggyColorRenderbuffer", "NoPBufferWithAccum",
-                                                          "NeedSharedObjectSync", "NoARBCreateContext", "NoSurfacelessCtx"
+                                                          "NeedSharedObjectSync", "NoARBCreateContext", "NoSurfacelessCtx",
+                                                          "NoFBOSupport", "DontChooseFBConfigBestMatch", "No10BitColorCompOffscreen"
                                                         };
 
     private static final IdentityHashMap<String, GLRendererQuirks> stickyDeviceQuirks = new IdentityHashMap<String, GLRendererQuirks>();
@@ -504,17 +594,6 @@ public class GLRendererQuirks {
         final GLRendererQuirks sq = getStickyDeviceQuirks(device);
         sq.addQuirk(quirk);
     }
-    /**
-     * {@link #addQuirks(int[], int, int) Adding given quirks} of sticky {@link AbstractGraphicsDevice}'s {@link GLRendererQuirks}.
-     * <p>
-     * Not thread safe.
-     * </p>
-     * @see #getStickyDeviceQuirks(AbstractGraphicsDevice)
-     */
-    public static void addStickyDeviceQuirks(final AbstractGraphicsDevice device, final int[] quirks, final int offset, final int len) throws IllegalArgumentException {
-        final GLRendererQuirks sq = getStickyDeviceQuirks(device);
-        sq.addQuirks(quirks, offset, len);
-    }
     /**
      * {@link #addQuirks(GLRendererQuirks) Adding given quirks} of sticky {@link AbstractGraphicsDevice}'s {@link GLRendererQuirks}.
      * <p>
@@ -533,8 +612,8 @@ public class GLRendererQuirks {
      * </p>
      * @see #getStickyDeviceQuirks(AbstractGraphicsDevice)
      */
-    public static boolean existStickyDeviceQuirk(final AbstractGraphicsDevice device, final int quirk) {
-        return getStickyDeviceQuirks(device).exist(quirk);
+    public static boolean existStickyDeviceQuirk(final AbstractGraphicsDevice device, final int quirkBit) {
+        return getStickyDeviceQuirks(device).exist(quirkBit);
     }
     /**
      * {@link #addQuirks(GLRendererQuirks) Pushing} the sticky {@link AbstractGraphicsDevice}'s {@link GLRendererQuirks}
@@ -548,6 +627,60 @@ public class GLRendererQuirks {
         dest.addQuirks(getStickyDeviceQuirks(device));
     }
 
+    public static final Override getOverride(final int quirkBit) throws IllegalArgumentException {
+        validateQuirk(quirkBit);
+        if( 0 != ( ( 1 << quirkBit )  & _bitmaskOverrideForce ) ) {
+            return Override.FORCE;
+        }
+        if( 0 != ( ( 1 << quirkBit )  & _bitmaskOverrideIgnore ) ) {
+            return Override.IGNORE;
+        }
+        return Override.NONE;
+    }
+    private static int _bitmaskOverrideForce = 0;
+    private static int _bitmaskOverrideIgnore = 0;
+    static {
+        _bitmaskOverrideForce = _queryQuirkMaskOfPropertyList("jogl.quirks.force", Override.FORCE);
+        _bitmaskOverrideIgnore = _queryQuirkMaskOfPropertyList("jogl.quirks.ignore", Override.IGNORE);
+        if( 0 != ( _bitmaskOverrideForce & GLRendererQuirks.BuggyColorRenderbuffer) ) {
+            _bitmaskOverrideForce |= GLRendererQuirks.NoFullFBOSupport;
+        }
+
+        final int uniqueTest = _bitmaskOverrideForce & _bitmaskOverrideIgnore;
+        if( 0 != uniqueTest ) {
+            throw new InternalError("Override properties force 0x"+Integer.toHexString(_bitmaskOverrideForce)+
+                                    " and ignore 0x"+Integer.toHexString(_bitmaskOverrideIgnore)+
+                                    " have intersecting bits 0x"+Integer.toHexString(uniqueTest)+" "+Integer.toBinaryString(uniqueTest));
+        }
+    }
+    private static int _queryQuirkMaskOfPropertyList(final String propertyName, final Override override) {
+        final String quirkNameList = PropertyAccess.getProperty(propertyName, true);
+        if( null == quirkNameList ) {
+            return 0;
+        }
+        int res = 0;
+        final String quirkNames[] = quirkNameList.split(",");
+        for(int i=0; i<quirkNames.length; i++) {
+            final String name = quirkNames[i].trim();
+            try {
+                final Field field = GLRendererQuirks.class.getField(name);
+                final int quirkBit = field.getInt(null);
+                final Override preOverride = getOverride(quirkBit);
+                if( Override.NONE != preOverride ) {
+                    System.err.println("Warning: Quirk '"+name+"' bit "+quirkBit+" skipped for override "+override+" mask, already set for override "+preOverride+". Has been included in given property "+propertyName);
+                } else {
+                    res |= 1 << quirkBit;
+                    System.err.println("Info: Quirk '"+name+"' bit "+quirkBit+" has been added to Override."+override+" mask as included in given property "+propertyName);
+                }
+            } catch (final NoSuchFieldException e) {
+                System.err.println("Warning: Failed to match given property "+propertyName+"'s quirk '"+name+"' with any supported quirk! "+e.getMessage());
+            } catch (SecurityException | IllegalAccessException e) {
+                System.err.println("Warning: Failed to access given property "+propertyName+"'s quirk '"+name+"' bit value! "+e.getMessage());
+            }
+        }
+        return res;
+    }
+
     private int _bitmask;
 
     public GLRendererQuirks() {
@@ -555,42 +688,12 @@ public class GLRendererQuirks {
     }
 
     /**
-     * @param quirks an array of valid quirks
-     * @param offset offset in quirks array to start reading
-     * @param len number of quirks to read from offset within quirks array
-     * @throws IllegalArgumentException if one of the quirks is out of range
-     */
-    public GLRendererQuirks(final int[] quirks, final int offset, final int len) throws IllegalArgumentException {
-        this();
-        addQuirks(quirks, offset, len);
-    }
-
-    /**
-     * @param quirk valid quirk to be added
+     * @param quirkBit valid quirk to be added
      * @throws IllegalArgumentException if the quirk is out of range
      */
-    public final void addQuirk(final int quirk) throws IllegalArgumentException {
-        validateQuirk(quirk);
-        _bitmask |= 1 << quirk;
-    }
-
-    /**
-     * @param quirks an array of valid quirks to be added
-     * @param offset offset in quirks array to start reading
-     * @param len number of quirks to read from offset within quirks array
-     * @throws IllegalArgumentException if one of the quirks is out of range
-     */
-    public final void addQuirks(final int[] quirks, final int offset, final int len) throws IllegalArgumentException {
-        int bitmask = 0;
-        if( !( 0 <= offset + len && offset + len <= quirks.length ) ) {
-            throw new IllegalArgumentException("offset and len out of bounds: offset "+offset+", len "+len+", array-len "+quirks.length);
-        }
-        for(int i=offset; i<offset+len; i++) {
-            final int quirk = quirks[i];
-            validateQuirk(quirk);
-            bitmask |= 1 << quirk;
-        }
-        _bitmask |= bitmask;
+    public final void addQuirk(final int quirkBit) throws IllegalArgumentException {
+        validateQuirk(quirkBit);
+        _bitmask |= 1 << quirkBit;
     }
 
     /**
@@ -601,13 +704,32 @@ public class GLRendererQuirks {
     }
 
     /**
-     * @param quirk the quirk to be tested
+     * Method tests whether the given quirk exists.
+     * <p>
+     * This methods respects the potential {@link GLRendererQuirks.Override}
+     * setting by user properties. Therefor this method returns {@code true}
+     * for {@code FORCE}'ed quirks and {@code false} for {@code IGNORE}'ed quirks.
+     * </p>
+     * @param quirkBit the quirk to be tested
      * @return true if quirk exist, otherwise false
      * @throws IllegalArgumentException if quirk is out of range
      */
-    public final boolean exist(final int quirk) throws IllegalArgumentException {
-        validateQuirk(quirk);
-        return 0 != ( ( 1 << quirk )  & _bitmask );
+    public final boolean exist(final int quirkBit) throws IllegalArgumentException {
+        validateQuirk(quirkBit);
+        return 0 != ( ( 1 << quirkBit )  & ( ~_bitmaskOverrideIgnore & ( _bitmask | _bitmaskOverrideForce ) ) );
+    }
+
+    /**
+     * Convenient static method to call {@link #exist(int)} on the given {@code quirks}
+     * with an added {@code null} check.
+     * @param quirks {@link GLRendererQuirks} instance, maybe {@code null}
+     * @param quirkBit the quirk to be tested
+     * @return {@code true} if the {@code quirks} is not {@code null} and the given {@code quirkBit} is set, otherwise {@code false}.
+     * @throws IllegalArgumentException if quirk is out of range
+     * @see #exist(int)
+     */
+    public static boolean exist(final GLRendererQuirks quirks, final int quirkBit) throws IllegalArgumentException {
+        return null != quirks && quirks.exist(quirkBit);
     }
 
     public final StringBuilder toString(StringBuilder sb) {
@@ -617,10 +739,14 @@ public class GLRendererQuirks {
         sb.append("[");
         boolean first=true;
         for(int i=0; i<getCount(); i++) {
-            final int testmask = 1 << i;
-            if( 0 != ( _bitmask & testmask ) ) {
+            // list ignored and forced as well
+            if( 0 != ( ( 1 << i ) & ( _bitmask | _bitmaskOverrideForce ) ) ) {
                 if(!first) { sb.append(", "); }
                 sb.append(toString(i));
+                final Override override = getOverride(i);
+                if( Override.NONE != override ) {
+                    sb.append("(").append(override.name().toLowerCase()).append("d)");
+                }
                 first=false;
             }
         }
@@ -628,28 +754,28 @@ public class GLRendererQuirks {
         return sb;
     }
 
-    @Override
+    // @Override
     public final String toString() {
         return toString(null).toString();
     }
 
     /**
-     * @param quirk the quirk to be validated, i.e. whether it is out of range
+     * @param quirkBit the quirk to be validated, i.e. whether it is out of range
      * @throws IllegalArgumentException if quirk is out of range
      */
-    public static void validateQuirk(final int quirk) throws IllegalArgumentException {
-        if( !( 0 <= quirk && quirk < getCount() ) ) {
-            throw new IllegalArgumentException("Quirks must be in range [0.."+getCount()+"[, but quirk: "+quirk);
+    public static void validateQuirk(final int quirkBit) throws IllegalArgumentException {
+        if( !( 0 <= quirkBit && quirkBit < getCount() ) ) {
+            throw new IllegalArgumentException("Quirks must be in range [0.."+getCount()+"[, but quirk: "+quirkBit);
         }
     }
 
     /**
-     * @param quirk the quirk to be converted to String
+     * @param quirkBit the quirk to be converted to String
      * @return the String equivalent of this quirk
      * @throws IllegalArgumentException if quirk is out of range
      */
-    public static final String toString(final int quirk) throws IllegalArgumentException {
-        validateQuirk(quirk);
-        return _names[quirk];
+    public static final String toString(final int quirkBit) throws IllegalArgumentException {
+        validateQuirk(quirkBit);
+        return _names[quirkBit];
     }
 }
