@@ -153,16 +153,20 @@ public class ImageSequence implements TextureSequence {
     private String textureLookupFunctionName = "myTexture2D";
 
     @Override
-    public String getTextureLookupFunctionName(final String desiredFuncName) throws IllegalStateException {
+    public String setTextureLookupFunctionName(final String texLookupFuncName) throws IllegalStateException {
         if(useBuildInTexLookup) {
-            return "texture2D";
-        }
-        if(null != desiredFuncName && desiredFuncName.length()>0) {
-            textureLookupFunctionName = desiredFuncName;
+            textureLookupFunctionName = "texture2D";
+        } else if(null != texLookupFuncName && texLookupFuncName.length()>0) {
+            textureLookupFunctionName = texLookupFuncName;
         }
         return textureLookupFunctionName;
     }
 
+    @Override
+    public String getTextureLookupFunctionName() throws IllegalStateException {
+        return textureLookupFunctionName;
+    }
+
     @Override
     public String getTextureLookupFragmentShaderImpl() throws IllegalStateException {
         if(useBuildInTexLookup) {
