@@ -1,5 +1,5 @@
 /**
- * Copyright 2010 JogAmp Community. All rights reserved.
+ * Copyright 2010-2023 JogAmp Community. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are
  * permitted provided that the following conditions are met:
@@ -30,6 +30,7 @@ package jogamp.graph.curve.opengl;
 import java.nio.FloatBuffer;
 
 import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLProfile;
 import com.jogamp.opengl.GL;
 import com.jogamp.opengl.GLUniformData;
 
@@ -45,15 +46,17 @@ import com.jogamp.graph.curve.opengl.RenderState;
 import com.jogamp.opengl.FBObject;
 import com.jogamp.opengl.FBObject.Attachment;
 import com.jogamp.opengl.FBObject.TextureAttachment;
-import com.jogamp.opengl.math.FloatUtil;
+import com.jogamp.opengl.math.Matrix4f;
+import com.jogamp.opengl.math.Recti;
 import com.jogamp.opengl.math.geom.AABBox;
 import com.jogamp.opengl.util.GLArrayDataServer;
+import com.jogamp.opengl.util.SyncMatrices4f16;
 import com.jogamp.opengl.util.glsl.ShaderProgram;
 import com.jogamp.opengl.util.texture.Texture;
 import com.jogamp.opengl.util.texture.TextureCoords;
 import com.jogamp.opengl.util.texture.TextureSequence;
 
-public class VBORegion2PVBAAES2  extends GLRegion {
+public final class VBORegion2PVBAAES2  extends GLRegion {
     private static final boolean DEBUG_FBO_1 = false;
     private static final boolean DEBUG_FBO_2 = false;
 
@@ -90,10 +93,6 @@ public class VBORegion2PVBAAES2  extends GLRegion {
     private final RenderState.ProgramLocal rsLocal;
 
     // Pass-1:
-    private GLArrayDataServer gca_VerticesAttr;
-    private GLArrayDataServer gca_CurveParamsAttr;
-    private GLArrayDataServer gca_ColorsAttr;
-    private GLArrayDataServer indicesBuffer;
     private final GLUniformData gcu_ColorTexUnit;
     private final float[] colorTexBBox; // x0, y0, x1, y1
     private final GLUniformData gcu_ColorTexBBox;
@@ -105,7 +104,8 @@ public class VBORegion2PVBAAES2  extends GLRegion {
     private GLArrayDataServer indicesFbo;
     private final GLUniformData gcu_FboTexUnit;
     private final GLUniformData gcu_FboTexSize;
-    private final float[] pmvMatrix02 = new float[2*16]; // P + Mv
+    private final Matrix4f matP = new Matrix4f();
+    private final Matrix4f matMv = new Matrix4f();
     private final GLUniformData gcu_PMVMatrix02;
     private ShaderProgram spPass2 = null;
 
@@ -121,19 +121,23 @@ public class VBORegion2PVBAAES2  extends GLRegion {
     /**
      * <p>
      * Since multiple {@link Region}s may share one
-     * {@link ShaderProgram}, the uniform data must always be updated.
+     * {@link ShaderProgram} managed and owned by {@link RegionRendered}, the uniform data must always be updated.
      * </p>
      *
      * @param gl
      * @param renderer
-     * @param renderModes
+     * @param curRenderModes
      * @param pass1
      * @param quality
      * @param sampleCount
      */
-    public void useShaderProgram(final GL2ES2 gl, final RegionRenderer renderer, final int renderModes, final boolean pass1, final int quality, final int sampleCount) {
+    public void useShaderProgram(final GL2ES2 gl, final RegionRenderer renderer, final int curRenderModes, final boolean pass1, final int quality, final int sampleCount) {
+        final boolean isTwoPass = Region.isTwoPass( curRenderModes );
+        final boolean hasColorChannel = Region.hasColorChannel( curRenderModes );
+        final boolean hasColorTexture = Region.hasColorTexture( curRenderModes ) && null != colorTexSeq;
+
         final RenderState rs = renderer.getRenderState();
-        final boolean updateLocGlobal = renderer.useShaderProgram(gl, renderModes, pass1, quality, sampleCount, colorTexSeq);
+        final boolean updateLocGlobal = renderer.useShaderProgram(gl, curRenderModes, pass1, quality, sampleCount, colorTexSeq);
         final ShaderProgram sp = renderer.getRenderState().getShaderProgram();
         final boolean updateLocLocal;
         if( pass1 ) {
@@ -145,13 +149,15 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             if( updateLocLocal ) {
                 rs.updateAttributeLoc(gl, true, gca_VerticesAttr, true);
                 rs.updateAttributeLoc(gl, true, gca_CurveParamsAttr, true);
-                if( null != gca_ColorsAttr ) {
+                if( hasColorChannel && null != gca_ColorsAttr ) {
                     rs.updateAttributeLoc(gl, true, gca_ColorsAttr, true);
                 }
             }
-            rsLocal.update(gl, rs, updateLocLocal, renderModes, true, true);
-            rs.updateUniformLoc(gl, updateLocLocal, gcu_PMVMatrix02, true);
-            if( null != gcu_ColorTexUnit ) {
+            rsLocal.update(gl, rs, updateLocLocal, curRenderModes, true, true);
+            if( isTwoPass ) {
+                rs.updateUniformLoc(gl, updateLocLocal, gcu_PMVMatrix02, true);
+            }
+            if( hasColorTexture && null != gcu_ColorTexUnit ) {
                 rs.updateUniformLoc(gl, updateLocLocal, gcu_ColorTexUnit, true);
                 rs.updateUniformLoc(gl, updateLocLocal, gcu_ColorTexBBox, true);
             }
@@ -165,33 +171,22 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                 rs.updateAttributeLoc(gl, true, gca_FboVerticesAttr, true);
                 rs.updateAttributeLoc(gl, true, gca_FboTexCoordsAttr, true);
             }
-            rsLocal.update(gl, rs, updateLocLocal, renderModes, false, true);
+            rsLocal.update(gl, rs, updateLocLocal, curRenderModes, false, true);
             rs.updateUniformDataLoc(gl, updateLocLocal, false /* updateData */, gcu_FboTexUnit, true); // FIXME always update if changing tex-unit
             rs.updateUniformLoc(gl, updateLocLocal, gcu_FboTexSize, sampleCount > 1); // maybe optimized away for sampleCount <= 1
         }
     }
 
-    public VBORegion2PVBAAES2(final int renderModes, final TextureSequence colorTexSeq, final int pass2TexUnit) {
-        super(renderModes, colorTexSeq);
+    public VBORegion2PVBAAES2(final GLProfile glp, final int renderModes, final TextureSequence colorTexSeq, final int pass2TexUnit,
+                              final int initialVerticesCount, final int initialIndicesCount)
+    {
+        super(glp, renderModes, colorTexSeq);
 
         rsLocal = new RenderState.ProgramLocal();
 
-        final int initialElementCount = 256;
-
         // Pass 1:
-        indicesBuffer = GLArrayDataServer.createData(3, GL.GL_SHORT, initialElementCount, GL.GL_STATIC_DRAW, GL.GL_ELEMENT_ARRAY_BUFFER);
+        initBuffer(initialVerticesCount, initialIndicesCount);
 
-        gca_VerticesAttr = GLArrayDataServer.createGLSL(AttributeNames.VERTEX_ATTR_NAME, 3, GL.GL_FLOAT,
-                                                      false, initialElementCount, GL.GL_STATIC_DRAW);
-
-        gca_CurveParamsAttr = GLArrayDataServer.createGLSL(AttributeNames.CURVEPARAMS_ATTR_NAME, 3, GL.GL_FLOAT,
-                                                       false, initialElementCount, GL.GL_STATIC_DRAW);
-        if( hasColorChannel() ) {
-            gca_ColorsAttr = GLArrayDataServer.createGLSL(AttributeNames.COLOR_ATTR_NAME, 4, GL.GL_FLOAT,
-                                                          false, initialElementCount, GL.GL_STATIC_DRAW);
-        } else {
-            gca_ColorsAttr = null;
-        }
         if( hasColorTexture() ) {
             gcu_ColorTexUnit = new GLUniformData(UniformNames.gcu_ColorTexUnit, colorTexSeq.getTextureUnit());
             colorTexBBox = new float[4];
@@ -201,16 +196,13 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             colorTexBBox = null;
             gcu_ColorTexBBox = null;
         }
-
-        FloatUtil.makeIdentity(pmvMatrix02, 0);
-        FloatUtil.makeIdentity(pmvMatrix02, 16);
-        gcu_PMVMatrix02 = new GLUniformData(UniformNames.gcu_PMVMatrix02, 4, 4, FloatBuffer.wrap(pmvMatrix02));
+        gcu_PMVMatrix02 = new GLUniformData(UniformNames.gcu_PMVMatrix02, 4, 4, new SyncMatrices4f16( new Matrix4f[] { matP, matMv } ));
 
         // Pass 2:
         gcu_FboTexUnit = new GLUniformData(UniformNames.gcu_FboTexUnit, pass2TexUnit);
         gcu_FboTexSize = new GLUniformData(UniformNames.gcu_FboTexSize, 2, FloatBuffer.wrap(new float[2]));
 
-        indicesFbo = GLArrayDataServer.createData(3, GL.GL_SHORT, 2, GL.GL_STATIC_DRAW, GL.GL_ELEMENT_ARRAY_BUFFER);
+        indicesFbo = GLArrayDataServer.createData(3, GL.GL_UNSIGNED_SHORT, 2, GL.GL_STATIC_DRAW, GL.GL_ELEMENT_ARRAY_BUFFER);
         indicesFbo.puts((short) 0); indicesFbo.puts((short) 1); indicesFbo.puts((short) 3);
         indicesFbo.puts((short) 1); indicesFbo.puts((short) 2); indicesFbo.puts((short) 3);
         indicesFbo.seal(true);
@@ -227,72 +219,27 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                                                            false, 4, GL.GL_STATIC_DRAW);
     }
 
+    @Override
+    public void setTextureUnit(final int pass2TexUnit) {
+        gcu_FboTexUnit.setData(pass2TexUnit);
+    }
+
     @Override
     protected final void clearImpl(final GL2ES2 gl) {
-        if(DEBUG_INSTANCE) {
-            System.err.println("VBORegion2PES2 Clear: " + this);
-            // Thread.dumpStack();
-        }
-        if( null != indicesBuffer ) {
-            indicesBuffer.seal(gl, false);
-            indicesBuffer.rewind();
-        }
-        if( null != gca_VerticesAttr ) {
-            gca_VerticesAttr.seal(gl, false);
-            gca_VerticesAttr.rewind();
-        }
-        if( null != gca_CurveParamsAttr ) {
-            gca_CurveParamsAttr.seal(gl, false);
-            gca_CurveParamsAttr.rewind();
-        }
-        if( null != gca_ColorsAttr ) {
-            gca_ColorsAttr.seal(gl, false);
-            gca_ColorsAttr.rewind();
-        }
         fboDirty = true;
     }
 
     @Override
-    protected final void pushVertex(final float[] coords, final float[] texParams, final float[] rgba) {
-        gca_VerticesAttr.putf(coords[0]);
-        gca_VerticesAttr.putf(coords[1]);
-        gca_VerticesAttr.putf(coords[2]);
+    protected void updateImpl(final GL2ES2 gl, final int curRenderModes) {
+        final boolean hasColorChannel = Region.hasColorChannel( curRenderModes );
+        final boolean hasColorTexture = Region.hasColorTexture( curRenderModes );
 
-        gca_CurveParamsAttr.putf(texParams[0]);
-        gca_CurveParamsAttr.putf(texParams[1]);
-        gca_CurveParamsAttr.putf(texParams[2]);
-
-        if( null != gca_ColorsAttr ) {
-            if( null != rgba ) {
-                gca_ColorsAttr.putf(rgba[0]);
-                gca_ColorsAttr.putf(rgba[1]);
-                gca_ColorsAttr.putf(rgba[2]);
-                gca_ColorsAttr.putf(rgba[3]);
-            } else {
-                throw new IllegalArgumentException("Null color given for COLOR_CHANNEL rendering mode");
-            }
-        }
-    }
-
-    @Override
-    protected final void pushIndex(final int idx) {
-        indicesBuffer.puts((short)idx);
-    }
-
-    @Override
-    protected void updateImpl(final GL2ES2 gl) {
         // seal buffers
         indicesBuffer.seal(gl, true);
         indicesBuffer.enableBuffer(gl, false);
-        gca_CurveParamsAttr.seal(gl, true);
-        gca_CurveParamsAttr.enableBuffer(gl, false);
-        gca_VerticesAttr.seal(gl, true);
-        gca_VerticesAttr.enableBuffer(gl, false);
-        if( null != gca_ColorsAttr ) {
-            gca_ColorsAttr.seal(gl, true);
-            gca_ColorsAttr.enableBuffer(gl, false);
-        }
-        if( null != gcu_ColorTexUnit && colorTexSeq.isTextureAvailable() ) {
+        vpc_ileave.seal(gl, true);
+        vpc_ileave.enableBuffer(gl, false);
+        if( hasColorTexture && null != gcu_ColorTexUnit && colorTexSeq.isTextureAvailable() ) {
             final TextureSequence.TextureFrame frame = colorTexSeq.getLastTexture();
             final Texture tex = frame.getTexture();
             final TextureCoords tc = tex.getImageTexCoords();
@@ -328,16 +275,13 @@ public class VBORegion2PVBAAES2  extends GLRegion {
     }
 
     private final AABBox drawWinBox = new AABBox();
-    private final int[] drawView = new int[] { 0, 0, 0, 0 };
-    private final float[] drawVec4Tmp0 = new float[4];
-    private final float[] drawVec4Tmp1 = new float[4];
-    private final float[] drawVec4Tmp2 = new float[4];
-    private final float[] drawMat4PMv = new float[16];
+    private final Recti drawView = new Recti();
+
     private static final int border = 2; // surrounding border, i.e. width += 2*border, height +=2*border
 
     @Override
-    protected void drawImpl(final GL2ES2 gl, final RegionRenderer renderer, final int[/*1*/] sampleCount) {
-        if( 0 >= indicesBuffer.getElementCount() ) {
+    protected void drawImpl(final GL2ES2 gl, final RegionRenderer renderer, final int curRenderModes, final int[/*1*/] sampleCount) {
+        if( 0 >= indicesBuffer.getElemCount() ) {
             if(DEBUG_INSTANCE) {
                 System.err.printf("VBORegion2PVBAAES2.drawImpl: Empty%n");
             }
@@ -351,8 +295,9 @@ public class VBORegion2PVBAAES2  extends GLRegion {
         }
         final int vpWidth = renderer.getWidth();
         final int vpHeight = renderer.getHeight();
-        if(vpWidth <=0 || vpHeight <= 0 || null==sampleCount || sampleCount[0] <= 0){
-            renderRegion(gl);
+        if(vpWidth <=0 || vpHeight <= 0 || null==sampleCount || sampleCount[0] <= 0) {
+            useShaderProgram(gl, renderer, curRenderModes, true, getQuality(), sampleCount[0]);
+            renderRegion(gl, curRenderModes);
         } else {
             if(0 > maxTexSize[0]) {
                 gl.glGetIntegerv(GL.GL_MAX_TEXTURE_SIZE, maxTexSize, 0);
@@ -370,12 +315,10 @@ public class VBORegion2PVBAAES2  extends GLRegion {
 
                 // Calculate perspective pixel width/height for FBO,
                 // considering the sampleCount.
-                drawView[2] = vpWidth;
-                drawView[3] = vpHeight;
+                drawView.setWidth(vpWidth);
+                drawView.setHeight(vpHeight);
 
-                renderer.getMatrix().multPMvMatrixf(drawMat4PMv, 0);
-                box.mapToWindow(drawWinBox, drawMat4PMv, drawView, true /* useCenterZ */,
-                                drawVec4Tmp0, drawVec4Tmp1, drawVec4Tmp2);
+                box.mapToWindow(drawWinBox, renderer.getMatrix().getPMvMat(), drawView, true /* useCenterZ */);
 
                 winWidth = drawWinBox.getWidth();
                 winHeight = drawWinBox.getHeight();
@@ -412,8 +355,8 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                     System.err.printf("XXX.MinMax obj d[%.3f, %.3f], r[%f, %f], b[%f, %f]%n",
                             diffObjWidth, diffObjHeight, ratioObjWinWidth, ratioObjWinWidth, diffObjBorderWidth, diffObjBorderHeight);
                     System.err.printf("XXX.MinMax win %s%n", drawWinBox.toString());
-                    System.err.printf("XXX.MinMax view[%d, %d] -> win[%.3f, %.3f], i[%d x %d], d[%.3f, %.3f], r[%f, %f]: FBO f[%.3f, %.3f], i[%d x %d], d[%.3f, %.3f], r[%f, %f], samples %d%n",
-                            drawView[2], drawView[3],
+                    System.err.printf("XXX.MinMax view[%s] -> win[%.3f, %.3f], i[%d x %d], d[%.3f, %.3f], r[%f, %f]: FBO f[%.3f, %.3f], i[%d x %d], d[%.3f, %.3f], r[%f, %f], samples %d%n",
+                            drawView,
                             winWidth, winHeight, targetWinWidth, targetWinHeight, diffWinWidth,
                             diffWinHeight, ratioWinWidth, ratioWinHeight,
                             renderFboWidth, renderFboHeight, targetFboWidth, targetFboHeight,
@@ -454,7 +397,7 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                     }
                     if( sampleCount[0] <= 0 ) {
                         // Last way out!
-                        renderRegion(gl);
+                        renderRegion(gl, curRenderModes);
                         return;
                     }
                 }
@@ -515,14 +458,14 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                     fb.position(12);
                 }
                 gca_FboVerticesAttr.seal(true);
-                FloatUtil.makeOrtho(pmvMatrix02, 0, true, minX, maxX, minY, maxY, -1, 1);
-                useShaderProgram(gl, renderer, getRenderModes(), true, getQuality(), sampleCount[0]);
-                renderRegion2FBO(gl, rs, targetFboWidth, targetFboHeight, newFboWidth, newFboHeight, vpWidth, vpHeight, sampleCount[0]);
+                matP.setToOrtho(minX, maxX, minY, maxY, -1, 1);
+                useShaderProgram(gl, renderer, curRenderModes, true, getQuality(), sampleCount[0]);
+                renderRegion2FBO(gl, rs, curRenderModes, targetFboWidth, targetFboHeight, newFboWidth, newFboHeight, vpWidth, vpHeight, sampleCount[0]);
             } else if( isStateDirty() ) {
-                useShaderProgram(gl, renderer, getRenderModes(), true, getQuality(), sampleCount[0]);
-                renderRegion2FBO(gl, rs, targetFboWidth, targetFboHeight, fboWidth, fboHeight, vpWidth, vpHeight, sampleCount[0]);
+                useShaderProgram(gl, renderer, curRenderModes, true, getQuality(), sampleCount[0]);
+                renderRegion2FBO(gl, rs, curRenderModes, targetFboWidth, targetFboHeight, fboWidth, fboHeight, vpWidth, vpHeight, sampleCount[0]);
             }
-            useShaderProgram(gl, renderer, getRenderModes(), false, getQuality(), sampleCount[0]);
+            useShaderProgram(gl, renderer, curRenderModes, false, getQuality(), sampleCount[0]);
             renderFBO(gl, rs, targetFboWidth, targetFboHeight, vpWidth, vpHeight, sampleCount[0]);
         }
     }
@@ -531,6 +474,11 @@ public class VBORegion2PVBAAES2  extends GLRegion {
                            final int vpWidth, final int vpHeight, final int sampleCount) {
         gl.glViewport(0, 0, vpWidth, vpHeight);
 
+        // Specific BITHINT_GLOBAL_DEPTH_TEST_ENABLED path added to
+        // VBORegion2PMSAA.renderVBO(..) by commit 45395696c252c215a8a22d05e5da7e98c662d07e (2014-04-10),
+        // but not here -> investigate if required for working MSAA:
+        // - "VBORegion2PMSAAES2 enables/disables GL_DEPTH_TEST, otherwise MSAA is corrupt"
+        //
         if( rs.isHintMaskSet(RenderState.BITHINT_BLENDING_ENABLED) ) {
             // RGB is already multiplied w/ alpha via renderRegion2FBO(..)
             gl.glBlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA);
@@ -545,7 +493,7 @@ public class VBORegion2PVBAAES2  extends GLRegion {
         gca_FboTexCoordsAttr.enableBuffer(gl, true);
         indicesFbo.bindBuffer(gl, true); // keeps VBO binding
 
-        gl.glDrawElements(GL.GL_TRIANGLES, indicesFbo.getElementCount() * indicesFbo.getComponentCount(), GL.GL_UNSIGNED_SHORT, 0);
+        gl.glDrawElements(GL.GL_TRIANGLES, indicesFbo.getElemCount() * indicesFbo.getCompsPerElem(), GL.GL_UNSIGNED_SHORT, 0);
 
         indicesFbo.bindBuffer(gl, false);
         gca_FboTexCoordsAttr.enableBuffer(gl, false);
@@ -555,7 +503,7 @@ public class VBORegion2PVBAAES2  extends GLRegion {
         // setback: gl.glActiveTexture(currentActiveTextureEngine[0]);
     }
 
-    private void renderRegion2FBO(final GL2ES2 gl, final RenderState rs,
+    private void renderRegion2FBO(final GL2ES2 gl, final RenderState rs, final int curRenderModes,
                                   final int targetFboWidth, final int targetFboHeight, final int newFboWidth, final int newFboHeight,
                                   final int vpWidth, final int vpHeight, final int sampleCount) {
         if( 0 >= targetFboWidth || 0 >= targetFboHeight ) {
@@ -616,22 +564,21 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
         }
 
-        renderRegion(gl);
+        renderRegion(gl, curRenderModes);
 
         fbo.unbind(gl);
         fboDirty = false;
     }
 
-    private void renderRegion(final GL2ES2 gl) {
+    private void renderRegion(final GL2ES2 gl, final int curRenderModes) {
+        final boolean hasColorChannel = Region.hasColorChannel( curRenderModes );
+        final boolean hasColorTexture = Region.hasColorTexture( curRenderModes );
+
         gl.glUniform(gcu_PMVMatrix02);
 
-        gca_VerticesAttr.enableBuffer(gl, true);
-        gca_CurveParamsAttr.enableBuffer(gl, true);
-        if( null != gca_ColorsAttr ) {
-            gca_ColorsAttr.enableBuffer(gl, true);
-        }
+        vpc_ileave.enableBuffer(gl, true);
         indicesBuffer.bindBuffer(gl, true); // keeps VBO binding
-        if( null != gcu_ColorTexUnit && colorTexSeq.isTextureAvailable() ) {
+        if( hasColorTexture && null != gcu_ColorTexUnit && colorTexSeq.isTextureAvailable() ) {
             final TextureSequence.TextureFrame frame = colorTexSeq.getNextTexture(gl);
             gl.glActiveTexture(GL.GL_TEXTURE0 + colorTexSeq.getTextureUnit());
             final Texture tex = frame.getTexture();
@@ -640,18 +587,14 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             gcu_ColorTexUnit.setData(colorTexSeq.getTextureUnit());
             gl.glUniform(gcu_ColorTexUnit); // Always update, since program maybe used by multiple regions
             gl.glUniform(gcu_ColorTexBBox); // Always update, since program maybe used by multiple regions
-            gl.glDrawElements(GL.GL_TRIANGLES, indicesBuffer.getElementCount() * indicesBuffer.getComponentCount(), GL.GL_UNSIGNED_SHORT, 0);
+            gl.glDrawElements(GL.GL_TRIANGLES, indicesBuffer.getElemCount() * indicesBuffer.getCompsPerElem(), glIdxType(), 0);
             tex.disable(gl); // nop on core
         } else {
-            gl.glDrawElements(GL.GL_TRIANGLES, indicesBuffer.getElementCount() * indicesBuffer.getComponentCount(), GL.GL_UNSIGNED_SHORT, 0);
+            gl.glDrawElements(GL.GL_TRIANGLES, indicesBuffer.getElemCount() * indicesBuffer.getCompsPerElem(), glIdxType(), 0);
         }
 
         indicesBuffer.bindBuffer(gl, false);
-        if( null != gca_ColorsAttr ) {
-            gca_ColorsAttr.enableBuffer(gl, false);
-        }
-        gca_CurveParamsAttr.enableBuffer(gl, false);
-        gca_VerticesAttr.enableBuffer(gl, false);
+        vpc_ileave.enableBuffer(gl, false);
     }
 
     @Override
@@ -665,22 +608,6 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             fbo = null;
             texA = null;
         }
-        if(null != gca_VerticesAttr) {
-            gca_VerticesAttr.destroy(gl);
-            gca_VerticesAttr = null;
-        }
-        if(null != gca_CurveParamsAttr) {
-            gca_CurveParamsAttr.destroy(gl);
-            gca_CurveParamsAttr = null;
-        }
-        if(null != gca_ColorsAttr) {
-            gca_ColorsAttr.destroy(gl);
-            gca_ColorsAttr = null;
-        }
-        if(null != indicesBuffer) {
-            indicesBuffer.destroy(gl);
-            indicesBuffer = null;
-        }
 
         if(null != gca_FboVerticesAttr) {
             gca_FboVerticesAttr.destroy(gl);
@@ -694,7 +621,7 @@ public class VBORegion2PVBAAES2  extends GLRegion {
             indicesFbo.destroy(gl);
             indicesFbo = null;
         }
-        spPass1 = null;
-        spPass2 = null;
+        spPass1 = null; // owned by RegionRenderer
+        spPass2 = null; // owned by RegionRenderer
     }
 }
