@@ -1,6 +1,4 @@
 /*
- * $Id: ResourceData.java,v 1.1.1.1 2004-12-05 23:14:31 davidsch Exp $
- *
  * Typecast - The Font Development Environment
  *
  * Copyright (c) 2004 David Schweinsberg
@@ -25,16 +23,15 @@ import java.io.IOException;
 
 /**
  *
- * @author <a href="mailto:davidsch@dev.java.net">David Schweinsberg</a>
- * @version $Id: ResourceData.java,v 1.1.1.1 2004-12-05 23:14:31 davidsch Exp $
+ * @author <a href="mailto:david.schweinsberg@gmail.com">David Schweinsberg</a>
  */
-public class ResourceData {
-
-    private final byte[] data;
+class ResourceData {
 
+    private byte[] data;
+    
     /** Creates new ResourceData */
-    public ResourceData(final DataInput di) throws IOException {
-        final int dataLen = di.readInt();
+    public ResourceData(DataInput di) throws IOException {
+        int dataLen = di.readInt();
         data = new byte[dataLen];
         di.readFully(data);
     }
