@@ -1,9 +1,9 @@
 /*****************************************************************************
  * Copyright (C) The Apache Software Foundation. All rights reserved.        *
- * ------------------------------------------------------------------------- *
- * This software is published under the terms of the Apache Software License *
- * version 1.1, a copy of which has been included with this distribution in  *
- * the LICENSE file.                                                         *
+ * ------------------------------------------------------------------------- * 
+ * This software is published under the terms of the Apache Software License * 
+ * version 1.1, a copy of which has been included with this distribution in  * 
+ * the LICENSE file.                                                         * 
  *****************************************************************************/
 
 package jogamp.graph.font.typecast.ot.table;
@@ -12,18 +12,17 @@ import java.io.DataInput;
 import java.io.IOException;
 
 /**
- * @version $Id: Program.java,v 1.1.1.1 2004-12-05 23:14:57 davidsch Exp $
- * @author <a href="mailto:davidsch@dev.java.net">David Schweinsberg</a>
+ * @author <a href="mailto:david.schweinsberg@gmail.com">David Schweinsberg</a>
  */
-public abstract class Program {
+abstract class Program {
 
     private short[] instructions;
 
-    public short[] getInstructions() {
+    short[] getInstructions() {
         return instructions;
     }
 
-    protected void readInstructions(final DataInput di, final int count) throws IOException {
+    void readInstructions(DataInput di, int count) throws IOException {
         instructions = new short[count];
         for (int i = 0; i < count; i++) {
             instructions[i] = (short) di.readUnsignedByte();
