@@ -31,25 +31,27 @@ package jogamp.opengl;
 import java.io.PrintStream;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.WindowClosingProtocol;
-import javax.media.nativewindow.WindowClosingProtocol.WindowClosingMode;
-import javax.media.opengl.FPSCounter;
-import javax.media.opengl.GL;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLOffscreenAutoDrawable;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.GLRunnable;
-import javax.media.opengl.GLSharedContextSetter;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.WindowClosingProtocol;
+import com.jogamp.nativewindow.WindowClosingProtocol.WindowClosingMode;
+import com.jogamp.opengl.FPSCounter;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLOffscreenAutoDrawable;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLRunnable;
+import com.jogamp.opengl.GLSharedContextSetter;
 
+import com.jogamp.common.ExceptionUtils;
+import com.jogamp.common.util.RunnableTask;
 import com.jogamp.common.util.locks.RecursiveLock;
 import com.jogamp.opengl.GLAutoDrawableDelegate;
 import com.jogamp.opengl.GLEventListenerState;
@@ -284,7 +286,7 @@ public abstract class GLAutoDrawableBase implements GLAutoDrawable, GLStateKeepe
             } catch( final Throwable t ) {
                 // Intentionally catch and ignore exception,
                 // so the destroy mechanism of the native windowing system is not corrupted!
-                GLException.dumpThrowable("ignored", t);
+                ExceptionUtils.dumpThrowable("ignored", t);
             }
         }
     }
@@ -597,6 +599,44 @@ public abstract class GLAutoDrawableBase implements GLAutoDrawable, GLStateKeepe
         return helper.getExclusiveContextThread();
     }
 
+    /**
+     * Invokes given {@code runnable} on current thread outside of a probable claimed exclusive thread,
+     * i.e. releases the exclusive thread, executes the runnable and reclaims it.
+     * FIXME: Promote to GLAutoDrawable!
+     *
+     * @param runnable the {@link Runnable} to execute on the new thread.
+     *                 The runnable <b>must exit</b>, i.e. not loop forever.
+     *
+     * @see #setExclusiveContextThread(Thread, GLContext)
+     *
+     * @since 2.3.2
+     */
+    public final void invokeOnCurrentThread(final Runnable runnable) {
+        helper.runOutsideOfExclusiveContextThread(context, runnable);
+    }
+    /**
+     * Invokes given {@code runnable} on current thread outside of a probable claimed exclusive thread,
+     * i.e. releases the exclusive thread, executes the runnable and reclaims it.
+     * FIXME: Promote to GLAutoDrawable!
+     *
+     * @param tg the {@link ThreadGroup} for the new thread, maybe <code>null</code>
+     * @param waitUntilDone if <code>true</code>, waits until <code>runnable</code> execution is completed, otherwise returns immediately.
+     * @param runnable the {@link Runnable} to execute on the new thread.
+     *                 The runnable <b>must exit</b>, i.e. not loop forever.
+     * @return {@link RunnableTask} instance with execution details
+     *
+     * @see #setExclusiveContextThread(Thread, GLContext)
+     *
+     * @since 2.3.2
+     */
+    public final RunnableTask invokeOnNewThread(final ThreadGroup tg, final boolean waitUntilDone, final Runnable runnable) {
+        return RunnableTask.invokeOnNewThread(tg, null, waitUntilDone,
+                new Runnable() {
+                    public final void run() {
+                        helper.runOutsideOfExclusiveContextThread(context, runnable);
+                    } });
+    }
+
     @Override
     public final boolean invoke(final boolean wait, final GLRunnable glRunnable) throws IllegalStateException {
         return helper.invoke(this, wait, glRunnable);
