@@ -48,7 +48,6 @@ import java.awt.Rectangle;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.security.AccessController;
 import java.security.PrivilegedAction;
 
 import com.jogamp.opengl.GL;
@@ -58,6 +57,7 @@ import com.jogamp.opengl.GLException;
 import com.jogamp.opengl.GLProfile;
 
 import com.jogamp.common.os.Platform;
+import com.jogamp.common.util.SecurityUtil;
 
 import jogamp.common.os.PlatformPropsImpl;
 import jogamp.opengl.Debug;
@@ -87,12 +87,14 @@ public class Java2D {
   private static Method getOGLSurfaceTypeMethod;
 
   // Publicly-visible constants for OpenGL surface types
-  public static final int UNDEFINED       = getOGLUtilitiesIntField("UNDEFINED");
-  public static final int WINDOW          = getOGLUtilitiesIntField("WINDOW");
-  public static final int PBUFFER         = getOGLUtilitiesIntField("PBUFFER");
-  public static final int TEXTURE         = getOGLUtilitiesIntField("TEXTURE");
-  public static final int FLIP_BACKBUFFER = getOGLUtilitiesIntField("FLIP_BACKBUFFER");
-  public static final int FBOBJECT        = getOGLUtilitiesIntField("FBOBJECT");
+  // Used to determine these by reflection, but in Java 9+ that's illegal, so had to just
+  // hard-code these from sun.java2d.pipe.hw.AccelSurface.
+  public static final int UNDEFINED       = 0;
+  public static final int WINDOW          = 1;
+  public static final int PBUFFER         = 2;  // this one doesn't even seem to exist in current versions of AccelSurface
+  public static final int TEXTURE         = 3;
+  public static final int FLIP_BACKBUFFER = 4;
+  public static final int FBOBJECT        = 5;
 
   // If FBOs are enabled in the Java2D/OpenGL pipeline, all contexts
   // created by JOGL must share textures and display lists with the
@@ -116,7 +118,7 @@ public class Java2D {
   private static Method destroyOGLContextMethod;
 
   static {
-    AccessController.doPrivileged(new PrivilegedAction<Object>() {
+    SecurityUtil.doPrivileged(new PrivilegedAction<Object>() {
         @Override
         public Object run() {
           if (DEBUG) {
@@ -565,31 +567,6 @@ public class Java2D {
     }
   }
 
-  private static int getOGLUtilitiesIntField(final String name) {
-    final Integer i = AccessController.doPrivileged(new PrivilegedAction<Integer>() {
-        @Override
-        public Integer run() {
-          try {
-            final Class<?> utils = Class.forName("sun.java2d.opengl.OGLUtilities");
-            final Field f = utils.getField(name);
-            f.setAccessible(true);
-            return (Integer) f.get(null);
-          } catch (final Exception e) {
-            if (DEBUG) {
-              e.printStackTrace();
-            }
-            return null;
-          }
-        }
-      });
-    if (i == null)
-      return 0;
-    if (DEBUG) {
-      System.err.println("OGLUtilities." + name + " = " + i.intValue());
-    }
-    return i.intValue();
-  }
-
   private static void initFBOShareContext(final GraphicsDevice device) {
     // Note 1: this must not be done in the static initalizer due to
     // deadlock problems.
