@@ -35,8 +35,6 @@ import com.jogamp.opengl.GL;
 import com.jogamp.opengl.GLException;
 import com.jogamp.opengl.GLProfile;
 
-import jogamp.opengl.util.av.GLMediaPlayerImpl;
-
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.os.Platform;
 import com.jogamp.common.util.IOUtil;
@@ -66,7 +64,7 @@ public class NullGLMediaPlayer extends GLMediaPlayerImpl {
     }
 
     @Override
-    protected final boolean playImpl() {
+    protected final boolean resumeImpl() {
         pos_start = Platform.currentTimeMillis();
         return true;
     }
@@ -76,6 +74,10 @@ public class NullGLMediaPlayer extends GLMediaPlayerImpl {
         return true;
     }
 
+    @Override
+    protected final void stopImpl() {
+    }
+
     @Override
     protected final int seekImpl(final int msec) {
         pos_ms = msec;
@@ -98,7 +100,7 @@ public class NullGLMediaPlayer extends GLMediaPlayerImpl {
     }
 
     @Override
-    protected final void destroyImpl(final GL gl) {
+    protected final void destroyImpl() {
         if(null != texData) {
             texData.destroy();
             texData = null;
@@ -108,7 +110,7 @@ public class NullGLMediaPlayer extends GLMediaPlayerImpl {
     public final static TextureData createTestTextureData() {
         TextureData res = null;
         try {
-            final URLConnection urlConn = IOUtil.getResource("jogl/util/data/av/test-ntsc01-28x16.png", NullGLMediaPlayer.class.getClassLoader());
+            final URLConnection urlConn = IOUtil.getResource("jogamp/opengl/assets/test-ntsc01-28x16.png", NullGLMediaPlayer.class.getClassLoader());
             if(null != urlConn) {
                 res = TextureIO.newTextureData(GLProfile.getGL2ES2(), urlConn.getInputStream(), false, TextureIO.PNG);
             }
