@@ -29,26 +29,28 @@ package jogamp.opengl.util.stereo;
 
 import java.nio.FloatBuffer;
 import java.nio.ShortBuffer;
+import java.util.Arrays;
 
-import javax.media.nativewindow.util.Dimension;
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.nativewindow.util.RectangleImmutable;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLArrayData;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLUniformData;
+import com.jogamp.nativewindow.util.Dimension;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.nativewindow.util.RectangleImmutable;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLArrayData;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLUniformData;
 
 import jogamp.common.os.PlatformPropsImpl;
 
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.os.Platform;
 import com.jogamp.opengl.JoglVersion;
+import com.jogamp.opengl.math.Vec3f;
 import com.jogamp.opengl.util.GLArrayDataServer;
 import com.jogamp.opengl.util.glsl.ShaderCode;
 import com.jogamp.opengl.util.glsl.ShaderProgram;
 import com.jogamp.opengl.util.stereo.EyeParameter;
-import com.jogamp.opengl.util.stereo.EyePose;
+import com.jogamp.opengl.util.stereo.ViewerPose;
 import com.jogamp.opengl.util.stereo.StereoDevice;
 import com.jogamp.opengl.util.stereo.StereoDeviceRenderer;
 import com.jogamp.opengl.util.stereo.StereoUtil;
@@ -81,7 +83,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
 
         private final EyeParameter eyeParameter;
 
-        private final EyePose eyePose;
 
         @Override
         public final RectangleImmutable getViewport() { return viewport; }
@@ -89,12 +90,9 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
         @Override
         public final EyeParameter getEyeParameter() { return eyeParameter; }
 
-        @Override
-        public final EyePose getLastEyePose() { return eyePose; }
-
-        private GenericEye(final GenericStereoDevice device, final int distortionBits,
-                         final float[] eyePositionOffset, final EyeParameter eyeParam,
-                         final DimensionImmutable textureSize, final RectangleImmutable eyeViewport) {
+        /* pp */ GenericEye(final GenericStereoDevice device, final int distortionBits,
+                            final Vec3f eyePositionOffset, final EyeParameter eyeParam,
+                            final DimensionImmutable textureSize, final RectangleImmutable eyeViewport) {
             this.eyeName = eyeParam.number;
             this.distortionBits = distortionBits;
             this.viewport = eyeViewport;
@@ -122,10 +120,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
 
             this.eyeParameter = eyeParam;
 
-            this.eyePose = new EyePose(eyeName);
-
-            updateEyePose(device); // 1st init
-
             // Setup: eyeToSourceUVScale, eyeToSourceUVOffset
             if( usePP ) {
                 final ScaleAndOffset2D textureScaleAndOffset = new ScaleAndOffset2D(eyeParam.fovhv, textureSize, eyeViewport);
@@ -332,18 +326,18 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
             indices.enableBuffer(gl, false);
         }
 
-        private void dispose(final GL2ES2 gl) {
+        /* pp */ void dispose(final GL2ES2 gl) {
             if( null == iVBO ) return;
             iVBO.destroy(gl);
             indices.destroy(gl);
         }
-        private void enableVBO(final GL2ES2 gl, final boolean enable) {
+        /* pp */ void enableVBO(final GL2ES2 gl, final boolean enable) {
             if( null == iVBO ) return;
             iVBO.enableBuffer(gl, enable);
             indices.bindBuffer(gl, enable); // keeps VBO binding if enable:=true
         }
 
-        private void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
+        /* pp */ void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
             if( null == iVBO ) return;
             gl.glUniform(eyeToSourceUVScale);
             gl.glUniform(eyeToSourceUVOffset);
@@ -353,15 +347,6 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
             }
         }
 
-        /**
-         * Updates {@link #ovrEyePose} and it's extracted
-         * {@link #eyeRenderPoseOrientation} and {@link #eyeRenderPosePosition}.
-         * @param hmdCtx used get the {@link #ovrEyePose} via {@link OVR#ovrHmd_GetEyePose(OvrHmdContext, int)}
-         */
-        private EyePose updateEyePose(final GenericStereoDevice hmdCtx) {
-            return eyePose;
-        }
-
         @Override
         public String toString() {
             final String ppTxt = null == iVBO ? ", no post-processing" :
@@ -372,15 +357,16 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
                         ", "+eyeParameter+
                         ", vertices "+vertexCount+", indices "+indexCount+
                         ppTxt+
-                        ", desc"+eyeParameter+", "+eyePose+"]";
+                        ", desc "+eyeParameter+"]";
         }
     }
 
     private final GenericStereoDevice device;
     private final GenericEye[] eyes;
+    private final ViewerPose viewerPose;
     private final int distortionBits;
     private final int textureCount;
-    private final DimensionImmutable singleTextureSize;
+    private final DimensionImmutable[] eyeTextureSizes;
     private final DimensionImmutable totalTextureSize;
     /** if texUnit0 is null: no post-processing */
     private final GLUniformData texUnit0;
@@ -391,7 +377,7 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
     @Override
     public String toString() {
         return "GenericStereo[distortion["+StereoUtil.distortionBitsToString(distortionBits)+
-                             "], singleSize "+singleTextureSize+
+                             "], eyeTexSize "+Arrays.toString(eyeTextureSizes)+
                              ", sbsSize "+totalTextureSize+
                              ", texCount "+textureCount+", texUnit "+(null != texUnit0 ? texUnit0.intValue() : "n/a")+
                              ", "+PlatformPropsImpl.NEWLINE+"  "+(0 < eyes.length ? eyes[0] : "none")+
@@ -402,15 +388,19 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
     private static final DimensionImmutable zeroSize = new Dimension(0, 0);
 
     /* pp */ GenericStereoDeviceRenderer(final GenericStereoDevice context, final int distortionBits,
-                                       final int textureCount, final float[] eyePositionOffset,
+                                       final int textureCount, final Vec3f eyePositionOffset,
                                        final EyeParameter[] eyeParam, final float pixelsPerDisplayPixel, final int textureUnit,
-                                       final DimensionImmutable singleTextureSize, final DimensionImmutable totalTextureSize,
+                                       final DimensionImmutable[] eyeTextureSizes, final DimensionImmutable totalTextureSize,
                                        final RectangleImmutable[] eyeViewports) {
+        if( eyeParam.length != eyeTextureSizes.length ||
+            eyeParam.length != eyeViewports.length ) {
+            throw new IllegalArgumentException("eye arrays of different length");
+        }
         this.device = context;
         this.eyes = new GenericEye[eyeParam.length];
         this.distortionBits = ( distortionBits | context.getMinimumDistortionBits() ) & context.getSupportedDistortionBits();
         final boolean usePP = null != device.config.distortionMeshProducer && 0 != this.distortionBits;
-        final DimensionImmutable textureSize;
+        final DimensionImmutable[] textureSizes;
 
         if( usePP ) {
             if( 1 > textureCount || 2 < textureCount ) {
@@ -418,20 +408,33 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
             } else {
                 this.textureCount = textureCount;
             }
-            this.singleTextureSize = singleTextureSize;
+            this.eyeTextureSizes = eyeTextureSizes;
             this.totalTextureSize = totalTextureSize;
-            textureSize = 1 == textureCount ? totalTextureSize : singleTextureSize;
+            if( 1 == textureCount ) {
+                textureSizes = new DimensionImmutable[eyeParam.length];
+                for(int i=0; i<eyeParam.length; i++) {
+                    textureSizes[i] = totalTextureSize;
+                }
+            } else {
+                textureSizes = eyeTextureSizes;
+            }
             texUnit0 = new GLUniformData("svr_Texture0", textureUnit);
         } else {
             this.textureCount = 0;
-            this.singleTextureSize = zeroSize;
+            this.eyeTextureSizes = new DimensionImmutable[eyeParam.length];
+            textureSizes = new DimensionImmutable[eyeParam.length];
+            for(int i=0; i<eyeParam.length; i++) {
+                this.eyeTextureSizes[i] = zeroSize;
+                textureSizes[i] = zeroSize;
+            }
             this.totalTextureSize = zeroSize;
-            textureSize = zeroSize;
             texUnit0 = null;
         }
+        viewerPose = new ViewerPose();
         for(int i=0; i<eyeParam.length; i++) {
-            eyes[i] = new GenericEye(context, this.distortionBits, eyePositionOffset, eyeParam[i], textureSize, eyeViewports[i]);
+            eyes[i] = new GenericEye(context, this.distortionBits, eyePositionOffset, eyeParam[i], textureSizes[i], eyeViewports[i]);
         }
+
         sp = null;
     }
 
@@ -445,7 +448,7 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
     public final boolean usesSideBySideStereo() { return true; }
 
     @Override
-    public final DimensionImmutable getSingleSurfaceSize() { return singleTextureSize; }
+    public final DimensionImmutable[] getEyeSurfaceSize() { return eyeTextureSizes; }
 
     @Override
     public final DimensionImmutable getTotalSurfaceSize() { return totalTextureSize; }
@@ -543,8 +546,14 @@ public class GenericStereoDeviceRenderer implements StereoDeviceRenderer {
     }
 
     @Override
-    public final EyePose updateEyePose(final int eyeNum) {
-        return eyes[eyeNum].updateEyePose(device);
+    public final ViewerPose updateViewerPose() {
+        // NOP
+        return viewerPose;
+    }
+
+    @Override
+    public final ViewerPose getLastViewerPose() {
+        return viewerPose;
     }
 
     @Override
