@@ -1,6 +1,6 @@
 /*
+ * Copyright (c) 2010-2023 JogAmp Community. All rights reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc. All Rights Reserved.
- * Copyright (c) 2010 JogAmp Community. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,6 +43,7 @@ import java.util.List;
 import java.util.Map;
 
 import com.jogamp.nativewindow.util.Point;
+
 import jogamp.common.os.PlatformPropsImpl;
 import jogamp.nativewindow.BcmVCArtifacts;
 import jogamp.nativewindow.Debug;
@@ -540,24 +541,6 @@ public abstract class NativeWindowFactory {
         return NativeWindowFactoryImpl.getNullToolkitLock();
     }
 
-    /**
-     * Provides the default {@link ToolkitLock} for <code>type</code> and <code>deviceHandle</code>.
-     * <ul>
-     *   <li> JAWT {@link ToolkitLock} if required and <code>type</code> is of {@link #TYPE_AWT} and AWT available,</li>
-     *   <li> {@link jogamp.nativewindow.ResourceToolkitLock} if required, otherwise</li>
-     *   <li> {@link jogamp.nativewindow.NullToolkitLock} </li>
-     * </ul>
-     */
-    public static ToolkitLock getDefaultToolkitLock(final String type, final long deviceHandle) {
-        if( requiresToolkitLock ) {
-            if( TYPE_AWT == type && isAWTAvailable() ) { // uses .intern()!
-                return getAWTToolkitLock();
-            }
-            return ResourceToolkitLock.create();
-        }
-        return NativeWindowFactoryImpl.getNullToolkitLock();
-    }
-
     /**
      * @param device
      * @param screen -1 is default screen of the given device, e.g. maybe 0 or determined by native API. >= 0 is specific screen
@@ -717,6 +700,7 @@ public abstract class NativeWindowFactory {
     public static AbstractGraphicsDevice createDevice(final String displayConnection, final boolean own) {
         return createDevice(NativeWindowFactory.getNativeWindowType(true), displayConnection, own);
     }
+
     /**
      * Creates a native device type, following the given {@link #getNativeWindowType(boolean) native-window-type}.
      * <p>
@@ -759,7 +743,7 @@ public abstract class NativeWindowFactory {
                     throw new NativeWindowException("EGLDisplayUtil.eglCreateEGLGraphicsDevice failed");
                 }
             } else {
-                device = new com.jogamp.nativewindow.egl.EGLGraphicsDevice(0, 0 /* EGL.EGL_NO_DISPLAY */, displayConnection, AbstractGraphicsDevice.DEFAULT_UNIT, null);
+                device = new com.jogamp.nativewindow.egl.EGLGraphicsDevice(0 /* EGL.EGL_DEFAULT_DISPLAY */, displayConnection, AbstractGraphicsDevice.DEFAULT_UNIT);
             }
             return device;
         } else if( NativeWindowFactory.TYPE_AWT == nwt ) {
