@@ -38,9 +38,6 @@ import java.awt.Canvas;
 import java.awt.Graphics;
 import java.awt.GraphicsDevice;
 import java.awt.GraphicsConfiguration;
-import java.lang.reflect.Method;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 
 import com.jogamp.nativewindow.AbstractGraphicsDevice;
 import com.jogamp.nativewindow.AbstractGraphicsScreen;
@@ -58,6 +55,8 @@ import com.jogamp.nativewindow.awt.AWTGraphicsScreen;
 import com.jogamp.nativewindow.awt.JAWTWindow;
 import com.jogamp.newt.Window;
 
+import jogamp.nativewindow.jawt.JAWTUtil;
+
 @SuppressWarnings("serial")
 public class AWTCanvas extends Canvas {
   private final WindowDriver driver;
@@ -122,7 +121,7 @@ public class AWTCanvas extends Canvas {
   public void addNotify() {
 
     // before native peer is valid: X11
-    disableBackgroundErase();
+    JAWTUtil.disableBackgroundErase(this);
 
     /**
      * 'super.addNotify()' determines the GraphicsConfiguration,
@@ -147,7 +146,7 @@ public class AWTCanvas extends Canvas {
     super.addNotify();
 
     // after native peer is valid: Windows
-    disableBackgroundErase();
+    JAWTUtil.disableBackgroundErase(this);
 
     {
         jawtWindow = (JAWTWindow) NativeWindowFactory.getNativeWindow(this, awtConfig);
@@ -347,57 +346,4 @@ public class AWTCanvas extends Canvas {
 
     return config;
   }
-
-  // Disables the AWT's erasing of this Canvas's background on Windows
-  // in Java SE 6. This internal API is not available in previous
-  // releases, but the system property
-  // -Dsun.awt.noerasebackground=true can be specified to get similar
-  // results globally in previous releases.
-  private static boolean disableBackgroundEraseInitialized;
-  private static Method  disableBackgroundEraseMethod;
-  private void disableBackgroundErase() {
-    if (!disableBackgroundEraseInitialized) {
-      try {
-        AccessController.doPrivileged(new PrivilegedAction<Object>() {
-            @Override
-            public Object run() {
-              try {
-                Class<?> clazz = getToolkit().getClass();
-                while (clazz != null && disableBackgroundEraseMethod == null) {
-                  try {
-                    disableBackgroundEraseMethod =
-                      clazz.getDeclaredMethod("disableBackgroundErase",
-                                              new Class[] { Canvas.class });
-                    disableBackgroundEraseMethod.setAccessible(true);
-                  } catch (final Exception e) {
-                    clazz = clazz.getSuperclass();
-                  }
-                }
-              } catch (final Exception e) {
-              }
-              return null;
-            }
-          });
-      } catch (final Exception e) {
-      }
-      disableBackgroundEraseInitialized = true;
-      if(Window.DEBUG_IMPLEMENTATION) {
-        System.err.println("AWTCanvas: TK disableBackgroundErase method found: "+
-                (null!=disableBackgroundEraseMethod));
-      }
-    }
-    if (disableBackgroundEraseMethod != null) {
-      Throwable t=null;
-      try {
-        disableBackgroundEraseMethod.invoke(getToolkit(), new Object[] { this });
-      } catch (final Exception e) {
-        // FIXME: workaround for 6504460 (incorrect backport of 6333613 in 5.0u10)
-        // throw new GLException(e);
-        t = e;
-      }
-      if(Window.DEBUG_IMPLEMENTATION) {
-        System.err.println("AWTCanvas: TK disableBackgroundErase error: "+t);
-      }
-    }
-  }
 }
