@@ -52,7 +52,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl {
     }
 
     @Override
-    protected void createNativeImpl() {
+    protected void createNativeImpl(boolean[] positionModified) {
         if(0!=getParentWindowHandle()) {
             throw new RuntimeException("Window parenting not supported (yet)");
         }
@@ -88,7 +88,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl {
             // n/a in BroadcomEGL
             System.err.println("BCEGL Window.setSizeImpl n/a in BroadcomEGL with realized window");
         } else {
-            defineSize(width, height);
+            defineWindowSize(width, height);
         }
     }
 
@@ -113,7 +113,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl {
                 // n/a in BroadcomEGL
                 System.err.println("BCEGL Window.setSizeImpl n/a in BroadcomEGL with realized window");
             } else {
-                defineSize((width>0)?width:getWidth(), (height>0)?height:getHeight());
+                defineWindowSize((width>0)?width:getWidth(), (height>0)?height:getHeight());
             }
         }
         if(x>=0 || y>=0) {
@@ -121,7 +121,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl {
         }
 
         if( 0 != ( CHANGE_MASK_VISIBILITY & flags) ) {
-            visibleChanged(false, 0 != ( STATE_MASK_VISIBLE & flags));
+            visibleChanged(0 != ( STATE_MASK_VISIBLE & flags));
         }
         return true;
     }
@@ -160,7 +160,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl {
     }
 
     private void windowCreated(final int cfgID, final int width, final int height) {
-        defineSize(width, height);
+        defineWindowSize(width, height);
         final GLCapabilitiesImmutable capsReq = (GLCapabilitiesImmutable) getGraphicsConfiguration().getRequestedCapabilities();
         final AbstractGraphicsConfiguration cfg = EGLGraphicsConfiguration.create(capsReq, getScreen().getGraphicsScreen(), cfgID);
         if (null == cfg) {
