@@ -29,39 +29,42 @@ package jogamp.opengl.oculusvr;
 
 import java.nio.FloatBuffer;
 import java.nio.ShortBuffer;
+import java.util.Arrays;
 
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.nativewindow.util.RectangleImmutable;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLArrayData;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLUniformData;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.nativewindow.util.RectangleImmutable;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLArrayData;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLUniformData;
 
 import jogamp.common.os.PlatformPropsImpl;
 
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.oculusvr.OVR;
 import com.jogamp.oculusvr.OVRException;
-import com.jogamp.oculusvr.OvrHmdContext;
 import com.jogamp.oculusvr.ovrDistortionMesh;
 import com.jogamp.oculusvr.ovrDistortionVertex;
 import com.jogamp.oculusvr.ovrEyeRenderDesc;
 import com.jogamp.oculusvr.ovrFovPort;
 import com.jogamp.oculusvr.ovrFrameTiming;
+import com.jogamp.oculusvr.ovrHmdDesc;
 import com.jogamp.oculusvr.ovrMatrix4f;
 import com.jogamp.oculusvr.ovrPosef;
 import com.jogamp.oculusvr.ovrRecti;
 import com.jogamp.oculusvr.ovrSizei;
+import com.jogamp.oculusvr.ovrTrackingState;
 import com.jogamp.oculusvr.ovrVector2f;
 import com.jogamp.oculusvr.ovrVector3f;
 import com.jogamp.opengl.JoglVersion;
 import com.jogamp.opengl.math.FloatUtil;
+import com.jogamp.opengl.math.Vec3f;
 import com.jogamp.opengl.util.GLArrayDataServer;
 import com.jogamp.opengl.util.glsl.ShaderCode;
 import com.jogamp.opengl.util.glsl.ShaderProgram;
 import com.jogamp.opengl.util.stereo.EyeParameter;
-import com.jogamp.opengl.util.stereo.EyePose;
+import com.jogamp.opengl.util.stereo.ViewerPose;
 import com.jogamp.opengl.util.stereo.StereoDevice;
 import com.jogamp.opengl.util.stereo.StereoDeviceRenderer;
 import com.jogamp.opengl.util.stereo.StereoUtil;
@@ -92,12 +95,11 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
         private final GLArrayData vboPos, vboParams, vboTexCoordsR, vboTexCoordsG, vboTexCoordsB;
         private final GLArrayDataServer indices;
 
-        private final ovrEyeRenderDesc ovrEyeDesc;
+        /* pp */ final ovrEyeRenderDesc ovrEyeDesc;
         private final ovrFovPort ovrEyeFov;
         private final EyeParameter eyeParameter;
 
-        private ovrPosef ovrEyePose;
-        private final EyePose eyePose;
+        private final ovrMatrix4f[] timeWarpMatrices;
 
         @Override
         public final RectangleImmutable getViewport() { return viewport; }
@@ -105,12 +107,9 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
         @Override
         public final EyeParameter getEyeParameter() { return eyeParameter; }
 
-        @Override
-        public final EyePose getLastEyePose() { return eyePose; }
-
-        private OVREye(final OvrHmdContext hmdCtx, final int distortionBits,
-                       final float[] eyePositionOffset, final ovrEyeRenderDesc eyeDesc,
-                       final ovrSizei ovrTextureSize, final RectangleImmutable eyeViewport) {
+        /* pp */ OVREye(final ovrHmdDesc hmdDesc, final int distortionBits,
+                        final Vec3f eyePositionOffset, final ovrEyeRenderDesc eyeDesc,
+                        final ovrSizei ovrTextureSize, final RectangleImmutable eyeViewport) {
             this.eyeName = eyeDesc.getEye();
             this.distortionBits = distortionBits;
             this.viewport = eyeViewport;
@@ -124,26 +123,26 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             if( usesTimewarp ) {
                 eyeRotationStart = new GLUniformData("ovr_EyeRotationStart", 4, 4, Buffers.slice2Float(fstash, 4, 16));
                 eyeRotationEnd = new GLUniformData("ovr_EyeRotationEnd", 4, 4, Buffers.slice2Float(fstash, 20, 16));
+                timeWarpMatrices = new ovrMatrix4f[2];
+                timeWarpMatrices[0] = ovrMatrix4f.create();
+                timeWarpMatrices[1] = ovrMatrix4f.create();
             } else {
                 eyeRotationStart = null;
                 eyeRotationEnd = null;
+                timeWarpMatrices = null;
             }
 
             this.ovrEyeDesc = eyeDesc;
             this.ovrEyeFov = eyeDesc.getFov();
 
-            final ovrVector3f eyeViewAdjust = eyeDesc.getViewAdjust();
+            final ovrVector3f eyeViewAdjust = eyeDesc.getHmdToEyeViewOffset();
             this.eyeParameter = new EyeParameter(eyeName, eyePositionOffset, OVRUtil.getFovHV(ovrEyeFov),
                                                  eyeViewAdjust.getX(), eyeViewAdjust.getY(), eyeViewAdjust.getZ());
 
-            this.eyePose = new EyePose(eyeName);
-
-            updateEyePose(hmdCtx); // 1st init
-
             // Setup: eyeToSourceUVScale, eyeToSourceUVOffset
             {
                 final ovrVector2f[] uvScaleOffsetOut = new ovrVector2f[2];
-                uvScaleOffsetOut[0] = ovrVector2f.create(); // FIXME: remove ctor / double check
+                uvScaleOffsetOut[0] = ovrVector2f.create();
                 uvScaleOffsetOut[1] = ovrVector2f.create();
 
                 final ovrRecti ovrEyeRenderViewport = OVRUtil.createOVRRecti(eyeViewport);
@@ -166,7 +165,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             final ovrDistortionMesh meshData = ovrDistortionMesh.create();
 
             final int ovrDistortionCaps = distBits2OVRDistCaps(distortionBits);
-            if( !OVR.ovrHmd_CreateDistortionMesh(hmdCtx, eyeName, ovrEyeFov, ovrDistortionCaps, meshData) ) {
+            if( !OVR.ovrHmd_CreateDistortionMesh(hmdDesc, eyeName, ovrEyeFov, ovrDistortionCaps, meshData) ) {
                 throw new OVRException("Failed to create meshData for eye "+eyeName+", "+OVRUtil.toString(ovrEyeFov)+" and "+StereoUtil.distortionBitsToString(distortionBits));
             }
             vertexCount = meshData.getVertexCount();
@@ -193,7 +192,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             /** 2+2+2+2+2: { vec2 position, vec2 color, vec2 texCoordR, vec2 texCoordG, vec2 texCoordB } */
             final FloatBuffer iVBOFB = (FloatBuffer)iVBO.getBuffer();
             final ovrDistortionVertex[] ovRes = new ovrDistortionVertex[1];
-            ovRes[0] = ovrDistortionVertex.create(); // FIXME: remove ctor / double check
+            ovRes[0] = ovrDistortionVertex.create();
 
             for ( int vertNum = 0; vertNum < vertexCount; vertNum++ ) {
                 final ovrDistortionVertex ov = meshData.getPVertexData(vertNum, ovRes)[0];
@@ -203,7 +202,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
                     System.err.println("XXX."+eyeName+": START VERTEX "+vertNum+" / "+vertexCount);
                 }
                 // pos
-                v = ov.getPos();
+                v = ov.getScreenPosNDC();
                 if( StereoDevice.DUMP_DATA ) {
                     System.err.println("XXX."+eyeName+": pos "+OVRUtil.toString(v));
                 }
@@ -225,7 +224,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
                 iVBOFB.put(ov.getTimeWarpFactor());
 
                 // texCoordR
-                v = ov.getTexR();
+                v = ov.getTanEyeAnglesR();
                 if( StereoDevice.DUMP_DATA ) {
                     System.err.println("XXX."+eyeName+": texR "+OVRUtil.toString(v));
                 }
@@ -234,7 +233,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
 
                 if( useChromatic ) {
                     // texCoordG
-                    v = ov.getTexG();
+                    v = ov.getTanEyeAnglesG();
                     if( StereoDevice.DUMP_DATA ) {
                         System.err.println("XXX."+eyeName+": texG "+OVRUtil.toString(v));
                     }
@@ -242,7 +241,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
                     iVBOFB.put(v.getY());
 
                     // texCoordB
-                    v = ov.getTexB();
+                    v = ov.getTanEyeAnglesB();
                     if( StereoDevice.DUMP_DATA ) {
                         System.err.println("XXX."+eyeName+": texB "+OVRUtil.toString(v));
                     }
@@ -274,7 +273,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             OVR.ovrHmd_DestroyDistortionMesh(meshData);
         }
 
-        private void linkData(final GL2ES2 gl, final ShaderProgram sp) {
+        /* pp */  void linkData(final GL2ES2 gl, final ShaderProgram sp) {
             if( 0 > vboPos.setLocation(gl, sp.program()) ) {
                 throw new GLException("Couldn't locate "+vboPos);
             }
@@ -312,16 +311,16 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             indices.enableBuffer(gl, false);
         }
 
-        private void dispose(final GL2ES2 gl) {
+        /* pp */  void dispose(final GL2ES2 gl) {
             iVBO.destroy(gl);
             indices.destroy(gl);
         }
-        private void enableVBO(final GL2ES2 gl, final boolean enable) {
+        /* pp */  void enableVBO(final GL2ES2 gl, final boolean enable) {
             iVBO.enableBuffer(gl, enable);
             indices.bindBuffer(gl, enable); // keeps VBO binding if enable:=true
         }
 
-        private void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
+        /* pp */ void updateUniform(final GL2ES2 gl, final ShaderProgram sp) {
             gl.glUniform(eyeToSourceUVScale);
             gl.glUniform(eyeToSourceUVOffset);
             if( StereoUtil.usesTimewarpDistortion(distortionBits) ) {
@@ -330,11 +329,8 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             }
         }
 
-        private void updateTimewarp(final OvrHmdContext hmdCtx, final ovrPosef eyeRenderPose, final float[] mat4Tmp1, final float[] mat4Tmp2) {
-            final ovrMatrix4f[] timeWarpMatrices = new ovrMatrix4f[2];
-            timeWarpMatrices[0] = ovrMatrix4f.create(); // FIXME: remove ctor / double check
-            timeWarpMatrices[1] = ovrMatrix4f.create();
-            OVR.ovrHmd_GetEyeTimewarpMatrices(hmdCtx, eyeName, eyeRenderPose, timeWarpMatrices);
+        /* pp */ void updateTimewarp(final ovrHmdDesc hmdDesc, final ovrPosef eyeRenderPose, final float[] mat4Tmp1, final float[] mat4Tmp2) {
+            OVR.ovrHmd_GetEyeTimewarpMatrices(hmdDesc, eyeName, eyeRenderPose, timeWarpMatrices);
 
             final float[] eyeRotationStartM = FloatUtil.transposeMatrix(timeWarpMatrices[0].getM(0, mat4Tmp1), mat4Tmp2);
             final FloatBuffer eyeRotationStartU = eyeRotationStart.floatBufferValue();
@@ -347,19 +343,6 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             eyeRotationEndU.rewind();
         }
 
-        /**
-         * Updates {@link #ovrEyePose} and it's extracted
-         * {@link #eyeRenderPoseOrientation} and {@link #eyeRenderPosePosition}.
-         * @param hmdCtx used get the {@link #ovrEyePose} via {@link OVR#ovrHmd_GetEyePose(OvrHmdContext, int)}
-         */
-        private EyePose updateEyePose(final OvrHmdContext hmdCtx) {
-            ovrEyePose = OVR.ovrHmd_GetEyePose(hmdCtx, eyeName);
-            final ovrVector3f pos = ovrEyePose.getPosition();
-            eyePose.setPosition(pos.getX(), pos.getY(), pos.getZ());
-            OVRUtil.copyToQuaternion(ovrEyePose.getOrientation(), eyePose.orientation);
-            return eyePose;
-        }
-
         @Override
         public String toString() {
             return "Eye["+eyeName+", viewport "+viewport+
@@ -367,15 +350,19 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
                         ", vertices "+vertexCount+", indices "+indexCount+
                         ", uvScale["+eyeToSourceUVScale.floatBufferValue().get(0)+", "+eyeToSourceUVScale.floatBufferValue().get(1)+
                         "], uvOffset["+eyeToSourceUVOffset.floatBufferValue().get(0)+", "+eyeToSourceUVOffset.floatBufferValue().get(1)+
-                        "], desc"+OVRUtil.toString(ovrEyeDesc)+", "+eyePose+"]";
+                        "], desc"+OVRUtil.toString(ovrEyeDesc)+"]";
         }
     }
 
     private final OVRStereoDevice context;
     private final OVREye[] eyes;
+    private final ovrPosef[] ovrEyePoses;
+    private final ovrVector3f[] hmdToEyeViewOffset;
+    private final ViewerPose viewerPose;
+    private final ovrTrackingState trackingState;
     private final int distortionBits;
     private final int textureCount;
-    private final DimensionImmutable singleTextureSize;
+    private final DimensionImmutable[] eyeTextureSizes;
     private final DimensionImmutable totalTextureSize;
     private final GLUniformData texUnit0;
 
@@ -385,11 +372,12 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
 
     private ShaderProgram sp;
     private ovrFrameTiming frameTiming;
+    private int frameCount;
 
     @Override
     public String toString() {
         return "OVRDist[distortion["+StereoUtil.distortionBitsToString(distortionBits)+
-                       "], singleSize "+singleTextureSize+
+                       "], eyeTexSize "+Arrays.toString(eyeTextureSizes)+
                        ", sbsSize "+totalTextureSize+
                        ", texCount "+textureCount+", texUnit "+getTextureUnit()+
                        ", "+PlatformPropsImpl.NEWLINE+"  "+eyes[0]+", "+PlatformPropsImpl.NEWLINE+"  "+eyes[1]+"]";
@@ -411,8 +399,9 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
     }
 
     /* pp */ OVRStereoDeviceRenderer(final OVRStereoDevice context, final int distortionBits,
-                             final int textureCount, final float[] eyePositionOffset,
-                             final ovrEyeRenderDesc[] eyeRenderDescs, final DimensionImmutable singleTextureSize, final DimensionImmutable totalTextureSize,
+                             final int textureCount, final Vec3f eyePositionOffset,
+                             final ovrEyeRenderDesc[] eyeRenderDescs,
+                             final DimensionImmutable[] eyeTextureSizes, final DimensionImmutable totalTextureSize,
                              final RectangleImmutable[] eyeViewports, final int textureUnit) {
         if( 1 > textureCount || 2 < textureCount ) {
             throw new IllegalArgumentException("textureCount can only be 1 or 2, has "+textureCount);
@@ -421,16 +410,34 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
         this.eyes = new OVREye[2];
         this.distortionBits = ( distortionBits | context.getMinimumDistortionBits() ) & context.getSupportedDistortionBits();
         this.textureCount = textureCount;
-        this.singleTextureSize = singleTextureSize;
+        this.eyeTextureSizes = eyeTextureSizes;
         this.totalTextureSize = totalTextureSize;
 
         texUnit0 = new GLUniformData("ovr_Texture0", textureUnit);
 
-        final ovrSizei ovrTextureSize = OVRUtil.createOVRSizei( 1 == textureCount ? totalTextureSize : singleTextureSize );
-        eyes[0] = new OVREye(context.handle, this.distortionBits, eyePositionOffset, eyeRenderDescs[0], ovrTextureSize, eyeViewports[0]);
-        eyes[1] = new OVREye(context.handle, this.distortionBits, eyePositionOffset, eyeRenderDescs[1], ovrTextureSize, eyeViewports[1]);
+        final ovrSizei ovrTexture0Size, ovrTexture1Size;
+        if( 1 == textureCount ) {
+            ovrTexture0Size = OVRUtil.createOVRSizei(totalTextureSize);
+            ovrTexture1Size = ovrTexture0Size;
+        } else {
+            ovrTexture0Size = OVRUtil.createOVRSizei(eyeTextureSizes[0]);
+            ovrTexture1Size = OVRUtil.createOVRSizei(eyeTextureSizes[1]);
+        }
+        eyes[0] = new OVREye(context.hmdDesc, this.distortionBits, eyePositionOffset, eyeRenderDescs[0], ovrTexture0Size, eyeViewports[0]);
+        eyes[1] = new OVREye(context.hmdDesc, this.distortionBits, eyePositionOffset, eyeRenderDescs[1], ovrTexture1Size, eyeViewports[1]);
+
+        ovrEyePoses = new ovrPosef[2];
+        ovrEyePoses[0] = ovrPosef.create();
+        ovrEyePoses[1] = ovrPosef.create();
+        hmdToEyeViewOffset = new ovrVector3f[2];
+        hmdToEyeViewOffset[0] = eyes[0].ovrEyeDesc.getHmdToEyeViewOffset();
+        hmdToEyeViewOffset[1] = eyes[1].ovrEyeDesc.getHmdToEyeViewOffset();
+        viewerPose = new ViewerPose();
+        trackingState = ovrTrackingState.create();
+
         sp = null;
         frameTiming = null;
+        frameCount = 0;
     }
 
     @Override
@@ -445,7 +452,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
     public final boolean usesSideBySideStereo() { return true; }
 
     @Override
-    public final DimensionImmutable getSingleSurfaceSize() { return singleTextureSize; }
+    public final DimensionImmutable[] getEyeSurfaceSize() { return eyeTextureSizes; }
 
     @Override
     public final DimensionImmutable getTotalSurfaceSize() { return totalTextureSize; }
@@ -535,13 +542,33 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
     }
 
     @Override
-    public final EyePose updateEyePose(final int eyeNum) {
-        return eyes[eyeNum].updateEyePose(context.handle);
+    public final ViewerPose updateViewerPose() {
+        final ovrTrackingState trackingState;
+        trackingState = this.trackingState;
+        OVR.ovrHmd_GetEyePoses(context.hmdDesc, frameCount, hmdToEyeViewOffset, ovrEyePoses, trackingState);
+        context.updateUsedSensorBits(trackingState);
+
+        // Use headPose of tracking state, since it points to the viewer
+        // where ovrEyePoses already have hmdToEyeViewOffset applied (IPD .. etc).
+        final ovrPosef pose = trackingState.getHeadPose().getThePose();
+        final ovrVector3f pos = pose.getPosition();
+        viewerPose.setPosition(pos.getX(), pos.getY(), pos.getZ());
+        OVRUtil.copyToQuaternion(pose.getOrientation(), viewerPose.orientation);
+
+        // System.err.println("Viewer: "+OVRUtil.toString(pos));
+        // System.err.println("Camera: "+OVRUtil.toString(trackingState.getCameraPose().getPosition()));
+        // System.err.println("Leveld: "+OVRUtil.toString(trackingState.getLeveledCameraPose().getPosition()));
+        return viewerPose;
+    }
+
+    @Override
+    public final ViewerPose getLastViewerPose() {
+        return viewerPose;
     }
 
     @Override
     public final void beginFrame(final GL gl) {
-        frameTiming = OVR.ovrHmd_BeginFrameTiming(context.handle, 0);
+        frameTiming = OVR.ovrHmd_BeginFrameTiming(context.hmdDesc, frameCount); // ovrHmd_GetFrameTiming not used, otherwise: frameCount);
     }
 
     @Override
@@ -549,8 +576,9 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
         if( null == frameTiming ) {
             throw new IllegalStateException("beginFrame not called");
         }
-        OVR.ovrHmd_EndFrameTiming(context.handle);
+        OVR.ovrHmd_EndFrameTiming(context.hmdDesc);
         frameTiming = null;
+        frameCount++;
     }
 
     @Override
@@ -562,7 +590,9 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
             throw new IllegalStateException("beginFrame not called");
         }
         if( StereoUtil.usesTimewarpDistortion(distortionBits) ) {
+            // minimize latency
             OVR.ovr_WaitTillTime(frameTiming.getTimewarpPointSeconds());
+            // {@link OVREye#updateTimewarp(ovrHmdDesc, ovrPosef, float[], float[])} will be called in {@link #ppOneEye}
         }
         final GL2ES2 gl2es2 = gl.getGL2ES2();
 
@@ -587,7 +617,7 @@ public class OVRStereoDeviceRenderer implements StereoDeviceRenderer {
     public final void ppOneEye(final GL gl, final int eyeNum) {
         final OVREye eye = eyes[eyeNum];
         if( StereoUtil.usesTimewarpDistortion(distortionBits) ) {
-            eye.updateTimewarp(context.handle, eye.ovrEyePose, mat4Tmp1, mat4Tmp2);
+            eye.updateTimewarp(context.hmdDesc, ovrEyePoses[eyeNum], mat4Tmp1, mat4Tmp2);
         }
         final GL2ES2 gl2es2 = gl.getGL2ES2();
 
