@@ -111,11 +111,17 @@ public class TestVersionSemanticsNOUI extends SingletonJunitCase {
     }
 
     @Test
-    public void testVersionV230V23x_00std() throws IllegalArgumentException, IOException, URISyntaxException {
+    public void testVersionV230V232() throws IllegalArgumentException, IOException, URISyntaxException {
+        testVersions(diffCriteria, Delta.CompatibilityType.BACKWARD_COMPATIBLE_BINARY, "2.3.0", "2.3.2", excludesStereoPackageAndAppletUtils);
+    }
+
+    @Test
+    public void testVersionV232V24x() throws IllegalArgumentException, IOException, URISyntaxException {
         final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.NON_BACKWARD_COMPATIBLE;
         // final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.BACKWARD_COMPATIBLE_USER;
+        // final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.BACKWARD_COMPATIBLE_BINARY;
 
-        final VersionNumberString preVersionNumber = new VersionNumberString("2.3.0");
+        final VersionNumberString preVersionNumber = new VersionNumberString("2.3.2");
         final File previousJar = new File("lib/v"+preVersionNumber.getVersionString()+"/"+jarFile);
 
         final ClassLoader currentCL = TestVersionSemanticsNOUI.class.getClassLoader();
@@ -125,22 +131,6 @@ public class TestVersionSemanticsNOUI extends SingletonJunitCase {
                                          curVersion.getClass(), currentCL, curVersionNumber,
                                          excludesDefault);
     }
-    @Test
-    public void testVersionV230V23x_01patch() throws IllegalArgumentException, IOException, URISyntaxException {
-        // final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.NON_BACKWARD_COMPATIBLE;
-        // final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.BACKWARD_COMPATIBLE_USER;
-        final Delta.CompatibilityType expectedCompatibilityType = Delta.CompatibilityType.BACKWARD_COMPATIBLE_BINARY;
-
-        final VersionNumberString preVersionNumber = new VersionNumberString("2.3.0");
-        final File previousJar = new File("lib/v"+preVersionNumber.getVersionString()+"/"+jarFile);
-
-        final ClassLoader currentCL = TestVersionSemanticsNOUI.class.getClassLoader();
-
-        VersionSemanticsUtil.testVersion(diffCriteria, expectedCompatibilityType,
-                                         previousJar, preVersionNumber,
-                                         curVersion.getClass(), currentCL, curVersionNumber,
-                                         excludesStereoPackageAndAppletUtils);
-    }
 
     public static void main(final String args[]) throws IOException {
         final String tstname = TestVersionSemanticsNOUI.class.getName();
