@@ -49,6 +49,8 @@ import com.jogamp.opengl.GLEventListenerState;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.GearsES2;
 import com.jogamp.opengl.test.junit.util.AWTRobotUtil;
 import com.jogamp.opengl.test.junit.util.GLEventListenerCounter;
+import com.jogamp.opengl.test.junit.util.GLTestUtil;
+import com.jogamp.opengl.test.junit.util.TestUtil;
 import com.jogamp.opengl.test.junit.util.UITestCase;
 
 import org.junit.Assert;
@@ -208,11 +210,11 @@ public abstract class GLContextDrawableSwitchBase1 extends UITestCase {
 
         if( GLADType.GLCanvasOnscreen == gladType || GLADType.GLCanvasOffscreen == gladType ) {
             setFrameVisible(frame);
-            Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, true));
+            Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, true, null));
         } else if( GLADType.GLWindow == gladType ) {
             ((GLWindow)glad).setVisible(true);
         }
-        Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glad, true));
+        Assert.assertEquals(true,  GLTestUtil.waitForRealized(glad, true, null));
         Assert.assertNotNull(glad.getContext());
         Assert.assertTrue(glad.isRealized());
 
@@ -235,10 +237,10 @@ public abstract class GLContextDrawableSwitchBase1 extends UITestCase {
             Assert.assertEquals(false, glelsIn.isOwner());
         }
 
-        for (int wait=0; wait<AWTRobotUtil.POLL_DIVIDER &&
+        for (int wait=0; wait<TestUtil.POLL_DIVIDER &&
                          ( 1 > glelTracker.initCount || 1 > glelTracker.reshapeCount || 1 > glelTracker.displayCount );
              wait++) {
-            Thread.sleep(AWTRobotUtil.TIME_SLICE);
+            Thread.sleep(TestUtil.TIME_SLICE);
         }
 
         final long t0 = System.currentTimeMillis();
@@ -270,13 +272,13 @@ public abstract class GLContextDrawableSwitchBase1 extends UITestCase {
         }
         if( GLADType.GLCanvasOnscreen == gladType || GLADType.GLCanvasOffscreen == gladType ) {
             destroyFrame(frame);
-            Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, false));
+            Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, false, null));
         } else if( GLADType.GLWindow == gladType ) {
             glad.destroy();
         } else if( GLADType.GLOffscreen == gladType ) {
             glad.destroy();
         }
-        Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glad, false));
+        Assert.assertEquals(true,  GLTestUtil.waitForRealized(glad, false, null));
 
         Assert.assertEquals(1, glelTracker.initCount);
         Assert.assertTrue(1 <= glelTracker.reshapeCount);
