@@ -116,7 +116,7 @@ public class FBOMix2DemosES2 implements GLEventListener {
         sp0.add(gl, fp0, System.err);
         st.attachShaderProgram(gl, sp0, true);
 
-        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.glGetPMvMatrixf());
+        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.getSyncPMvMat());
         st.ownUniform(pmvMatrixUniform);
         st.uniform(gl, pmvMatrixUniform);
 
