@@ -32,9 +32,9 @@ import com.jogamp.newt.event.PinchToZoomGesture;
 import com.jogamp.newt.event.GestureHandler.GestureEvent;
 import com.jogamp.opengl.GLRendererQuirks;
 import com.jogamp.opengl.JoglVersion;
-import com.jogamp.opengl.math.FloatUtil;
+import com.jogamp.opengl.math.Matrix4f;
 import com.jogamp.opengl.math.Quaternion;
-import com.jogamp.opengl.math.VectorUtil;
+import com.jogamp.opengl.math.Vec3f;
 import com.jogamp.opengl.test.junit.jogl.demos.GearsObject;
 import com.jogamp.opengl.util.CustomGLEventListener;
 import com.jogamp.opengl.util.PMVMatrix;
@@ -231,9 +231,9 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
         // Use debug pipeline
         // drawable.setGL(new DebugGL(drawable.getGL()));
 
-        pmvMatrix = new PMVMatrix();
+        pmvMatrix = new PMVMatrix(PMVMatrix.INVERSE_MODELVIEW | PMVMatrix.INVERSE_TRANSPOSED_MODELVIEW);
         st.attachObject("pmvMatrix", pmvMatrix);
-        pmvMatrixUniform = new GLUniformData("pmvMatrix", 4, 4, pmvMatrix.glGetPMvMvitMatrixf()); // P, Mv, Mvi and Mvit
+        pmvMatrixUniform = new GLUniformData("pmvMatrix", 4, 4, pmvMatrix.getSyncPMvMviMvitMat()); // P, Mv, Mvi and Mvit
         st.ownUniform(pmvMatrixUniform);
         st.uniform(gl, pmvMatrixUniform);
 
@@ -405,49 +405,59 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
     }
     // private boolean useAndroidDebug = false;
 
-    private final float[] mat4Tmp1 = new float[16];
-    private final float[] mat4Tmp2 = new float[16];
-    private final float[] vec3Tmp1 = new float[3];
-    private final float[] vec3Tmp2 = new float[3];
-    private final float[] vec3Tmp3 = new float[3];
+    private final Matrix4f mat4Tmp1 = new Matrix4f();
+    private final Matrix4f mat4Tmp2 = new Matrix4f();
+    private final Vec3f vec3Tmp1 = new Vec3f();
+    private final Vec3f vec3Tmp2 = new Vec3f();
+    private final Vec3f vec3Tmp3 = new Vec3f();
 
-    private static final float[] vec3ScalePos = new float[] { 20f, 20f, 20f };
+    private static final float scalePos = 20f;
 
     @Override
     public void reshapeForEye(final GLAutoDrawable drawable, final int x, final int y, final int width, final int height,
                               final EyeParameter eyeParam, final ViewerPose viewerPose) {
         final GL2ES2 gl = drawable.getGL().getGL2ES2();
-        pmvMatrix.glMatrixMode(GLMatrixFunc.GL_PROJECTION);
-        final float[] mat4Projection = FloatUtil.makePerspective(mat4Tmp1, 0, true, eyeParam.fovhv, zNear, zFar);
-        if( flipVerticalInGLOrientation && gl.getContext().getGLDrawable().isGLOriented() ) {
-            pmvMatrix.glLoadIdentity();
-            pmvMatrix.glScalef(1f, -1f, 1f);
-            pmvMatrix.glMultMatrixf(mat4Projection, 0);
-        } else {
-            pmvMatrix.glLoadMatrixf(mat4Projection, 0);
+
+        {
+            //
+            // Projection
+            //
+            final Matrix4f mat4 = new Matrix4f();
+            pmvMatrix.glMatrixMode(GLMatrixFunc.GL_PROJECTION);
+            if( flipVerticalInGLOrientation && gl.getContext().getGLDrawable().isGLOriented() ) {
+                mat4Tmp1.setToScale(1f, -1f, 1f);
+                mat4Tmp2.setToPerspective(eyeParam.fovhv, zNear, zFar);
+                mat4.mul(mat4Tmp1, mat4Tmp2);
+
+            } else {
+                mat4.setToPerspective(eyeParam.fovhv, zNear, zFar);
+            }
+            pmvMatrix.glLoadMatrixf(mat4);
+
+            //
+            // Modelview
+            //
+            pmvMatrix.glMatrixMode(GLMatrixFunc.GL_MODELVIEW);
+            final Quaternion rollPitchYaw = new Quaternion();
+            // private final float eyeYaw = FloatUtil.PI; // 180 degrees in radians
+            // rollPitchYaw.rotateByAngleY(eyeYaw);
+            // final Vec3f shiftedEyePos = rollPitchYaw.rotateVector(viewerPose.position, vec3Tmp1);
+            final Vec3f shiftedEyePos = vec3Tmp1.set(viewerPose.position);
+            shiftedEyePos.scale(scalePos); // amplify viewerPose position
+            shiftedEyePos.add(eyeParam.positionOffset);
+
+            rollPitchYaw.mult(viewerPose.orientation);
+            final Vec3f up = rollPitchYaw.rotateVector(Vec3f.UNIT_Y, vec3Tmp2);
+            final Vec3f forward = rollPitchYaw.rotateVector(Vec3f.UNIT_Z_NEG, vec3Tmp3); // -> center
+            final Vec3f center = forward.add(shiftedEyePos);
+
+            final Matrix4f mLookAt = mat4Tmp2.setToLookAt(shiftedEyePos, center, up, mat4Tmp1);
+            mat4.mul( mat4Tmp1.setToTranslation( eyeParam.distNoseToPupilX,
+                                                 eyeParam.distMiddleToPupilY,
+                                                 eyeParam.eyeReliefZ ), mLookAt);
+            mat4.translate(0, 0, -zViewDist, mat4Tmp1);
+            pmvMatrix.glLoadMatrixf(mat4);
         }
-
-        pmvMatrix.glMatrixMode(GLMatrixFunc.GL_MODELVIEW);
-
-        final Quaternion rollPitchYaw = new Quaternion();
-        // private final float eyeYaw = FloatUtil.PI; // 180 degrees in radians
-        // rollPitchYaw.rotateByAngleY(eyeYaw);
-        // final float[] shiftedEyePos = rollPitchYaw.rotateVector(vec3Tmp1, 0, viewerPose.position, 0);
-        final float[] shiftedEyePos = VectorUtil.copyVec3(vec3Tmp1, 0, viewerPose.position, 0);
-        VectorUtil.scaleVec3(shiftedEyePos, shiftedEyePos, vec3ScalePos); // amplify viewerPose position
-        VectorUtil.addVec3(shiftedEyePos, shiftedEyePos, eyeParam.positionOffset);
-
-        rollPitchYaw.mult(viewerPose.orientation);
-        final float[] up = rollPitchYaw.rotateVector(vec3Tmp2, 0, VectorUtil.VEC3_UNIT_Y, 0);
-        final float[] forward = rollPitchYaw.rotateVector(vec3Tmp3, 0, VectorUtil.VEC3_UNIT_Z_NEG, 0);
-        final float[] center = VectorUtil.addVec3(forward, shiftedEyePos, forward);
-
-        final float[] mLookAt = FloatUtil.makeLookAt(mat4Tmp1, 0, shiftedEyePos, 0, center, 0, up, 0, mat4Tmp2);
-        final float[] mViewAdjust = FloatUtil.makeTranslation(mat4Tmp2, true, eyeParam.distNoseToPupilX, eyeParam.distMiddleToPupilY, eyeParam.eyeReliefZ);
-        final float[] mat4Modelview = FloatUtil.multMatrix(mViewAdjust, mLookAt);
-
-        pmvMatrix.glLoadMatrixf(mat4Modelview, 0);
-        pmvMatrix.glTranslatef(0.0f, 0.0f, -zViewDist);
         st.useProgram(gl, true);
         st.uniform(gl, pmvMatrixUniform);
         st.useProgram(gl, false);
@@ -592,6 +602,7 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
     }
 
     class GearsKeyAdapter extends KeyAdapter {
+        @Override
         public void keyPressed(final KeyEvent e) {
             final int kc = e.getKeyCode();
             if(KeyEvent.VK_LEFT == kc) {
@@ -636,6 +647,7 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
             }
         }
 
+        @Override
         public void mousePressed(final MouseEvent e) {
             if( e.getPointerCount()==1 ) {
                 prevMouseX = e.getX();
@@ -648,9 +660,11 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
             }
         }
 
+        @Override
         public void mouseReleased(final MouseEvent e) {
         }
 
+        @Override
         public void mouseMoved(final MouseEvent e) {
             if( e.isConfined() ) {
                 navigate(e);
@@ -662,6 +676,7 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
             }
         }
 
+        @Override
         public void mouseDragged(final MouseEvent e) {
             navigate(e);
         }
