@@ -138,7 +138,7 @@ public class TextureDraw01ES2Listener implements GLEventListener, TextureDraw01A
         pmvMatrix.glLoadIdentity();
         pmvMatrix.glMatrixMode(GLMatrixFunc.GL_MODELVIEW);
         pmvMatrix.glLoadIdentity();
-        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.glGetPMvMatrixf()); // P, Mv
+        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.getSyncPMvMat()); // P, Mv
 
         st.ownUniform(pmvMatrixUniform);
         if(!st.uniform(gl, pmvMatrixUniform)) {
