@@ -84,12 +84,15 @@ public class TextureSequenceCubeES2 implements GLEventListener {
         int ly = 0;
         boolean first = false;
 
+        @Override
         public void mousePressed(final MouseEvent e) {
             first = true;
         }
+        @Override
         public void mouseMoved(final MouseEvent e) {
             first = false;
         }
+        @Override
         public void mouseDragged(final MouseEvent e) {
             int width, height;
             final Object source = e.getSource();
@@ -143,6 +146,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
                 ly = ny;
             }
         }
+        @Override
         public void mouseWheelMoved(final MouseEvent e) {
             // System.err.println("XXX "+e);
             if( !e.isShiftDown() ) {
@@ -182,7 +186,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
         rsFpPos = rsFp.insertShaderSource(0, rsFpPos, texSeq.getRequiredExtensionsShaderStub());
         rsFp.addDefaultShaderPrecision(gl, rsFpPos);
 
-        final String texLookupFuncName = texSeq.getTextureLookupFunctionName(myTextureLookupName);
+        final String texLookupFuncName = texSeq.setTextureLookupFunctionName(myTextureLookupName);
         rsFp.replaceInShaderSource(myTextureLookupName, texLookupFuncName);
 
         // Inject TextureSequence shader details
@@ -206,6 +210,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
 
     GLArrayDataServer interleavedVBO, cubeIndicesVBO;
 
+    @Override
     public void init(final GLAutoDrawable drawable) {
         final GL2ES2 gl = drawable.getGL().getGL2ES2();
         System.err.println(JoglVersion.getGLInfo(gl, null));
@@ -222,7 +227,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
 
         pmvMatrix = new PMVMatrix();
         reshapePMV(drawable.getSurfaceWidth(), drawable.getSurfaceHeight());
-        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.glGetPMvMatrixf()); // P, Mv
+        pmvMatrixUniform = new GLUniformData("mgl_PMVMatrix", 4, 4, pmvMatrix.getSyncPMvMat()); // P, Mv
         if(!st.uniform(gl, pmvMatrixUniform)) {
             throw new GLException("Error setting PMVMatrix in shader: "+st);
         }
@@ -304,6 +309,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
         System.out.println(st);
     }
 
+    @Override
     public void reshape(final GLAutoDrawable drawable, final int x, final int y, final int width, final int height) {
         final GL2ES2 gl = drawable.getGL().getGL2ES2();
 
@@ -397,7 +403,7 @@ public class TextureSequenceCubeES2 implements GLEventListener {
             }
         }
         cubeIndicesVBO.bindBuffer(gl, true); // keeps VBO binding
-        gl.glDrawElements(GL.GL_TRIANGLES, cubeIndicesVBO.getElementCount() * cubeIndicesVBO.getComponentCount(), GL.GL_UNSIGNED_SHORT, 0);
+        gl.glDrawElements(GL.GL_TRIANGLES, cubeIndicesVBO.getElemCount() * cubeIndicesVBO.getCompsPerElem(), GL.GL_UNSIGNED_SHORT, 0);
         cubeIndicesVBO.bindBuffer(gl, false);
 
         if(null != tex) {
