@@ -126,7 +126,7 @@ public class TestNewtKeyEventOrderAWT extends UITestCase {
                 frame1.setVisible(true);
             } } );
 
-        Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame1, true));
+        Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame1, true, null));
 
         testImpl(glWindow);
 
@@ -210,7 +210,7 @@ public class TestNewtKeyEventOrderAWT extends UITestCase {
         glWindow1KA.setVerbose(false);
         glWindow.addKeyListener(glWindow1KA);
 
-        Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glWindow, true));
+        Assert.assertEquals(true,  NewtTestUtil.waitForRealized(glWindow, true, null));
 
         // Continuous animation ..
         final Animator animator = new Animator(glWindow);
