@@ -50,7 +50,7 @@ import com.jogamp.newt.MonitorMode;
 import com.jogamp.newt.opengl.GLWindow;
 import com.jogamp.newt.util.MonitorModeUtil;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.GearsES2;
-import com.jogamp.opengl.test.junit.util.AWTRobotUtil;
+import com.jogamp.opengl.test.junit.util.NewtTestUtil;
 import com.jogamp.opengl.test.junit.util.UITestCase;
 
 import java.util.List;
@@ -142,7 +142,7 @@ public class TestScreenMode01dNEWT extends UITestCase {
     static void destroyWindow(final Window window) throws InterruptedException {
         if(null!=window) {
             window.destroy();
-            Assert.assertTrue(AWTRobotUtil.waitForRealized(window, false));
+            Assert.assertTrue(NewtTestUtil.waitForRealized(window, false, null));
         }
     }
 
@@ -189,7 +189,7 @@ public class TestScreenMode01dNEWT extends UITestCase {
         Assert.assertEquals(false,window.isVisible());
         Assert.assertEquals(false,window.isRealized());
         Assert.assertEquals(false,window.isNativeValid());
-        Assert.assertTrue(AWTRobotUtil.waitForRealized(screen, false));
+        Assert.assertTrue(NewtTestUtil.waitForRealized(screen, false, null));
         Assert.assertEquals(false,screen.isNativeValid());
         Assert.assertEquals(false,display.isNativeValid());
 
@@ -279,7 +279,7 @@ public class TestScreenMode01dNEWT extends UITestCase {
         Assert.assertEquals(false,window.isVisible());
         Assert.assertEquals(false,window.isRealized());
         Assert.assertEquals(false,window.isNativeValid());
-        Assert.assertTrue(AWTRobotUtil.waitForRealized(screen, false));
+        Assert.assertTrue(NewtTestUtil.waitForRealized(screen, false, null));
         Assert.assertEquals(false,screen.isNativeValid());
         Assert.assertEquals(false,display.isNativeValid());
 
@@ -386,7 +386,7 @@ public class TestScreenMode01dNEWT extends UITestCase {
         Assert.assertEquals(false,window.isVisible());
         Assert.assertEquals(false,window.isRealized());
         Assert.assertEquals(false,window.isNativeValid());
-        Assert.assertTrue(AWTRobotUtil.waitForRealized(screen, false));
+        Assert.assertTrue(NewtTestUtil.waitForRealized(screen, false, null));
         Assert.assertEquals(false,screen.isNativeValid());
         Assert.assertEquals(false,display.isNativeValid());
 
